@@ -107,7 +107,8 @@ int medusaConnectInternal(unsigned long nHost, int nPort, int nProtocol, int nTy
   char *buf, *tmpptr = NULL;
   char out[16];
   long flag;
-  int nOpt, nSize;
+  int nOpt;
+  unsigned int nSize;
   fd_set myset; 
   struct timeval tv;
   int nUseProxy = nProxyStringIP > 0 ? 1 : 0;
@@ -245,6 +246,35 @@ int medusaConnectInternal(unsigned long nHost, int nPort, int nProtocol, int nTy
     } 
     ret = s;
 
+    /*
+    // Possible issue with MTU/MSS values and our use of fixed buffer sizes.
+    // Example: OWA/EWS HTTPS
+    // We set a MTU of 616 (MSS of 576) in our SYN packet
+    // Windows responds with a MSS value of 536
+    // Windows sends a non-fragmented packet of 955 bytes
+    // Our buffer of 576 has issues... moving to a buffer of 1500 for now...
+   
+      int option = 576 + 40;
+      int optlen = sizeof(option);
+ 
+      // getsockopt call doesn't seem to work... use "ifconfig eth0 mtu 616" 
+
+      if (setsockopt(s, SOL_IP, IP_MTU, &option, optlen) < 0)
+        writeError(ERR_ERROR, "Error setting MTU size.");
+   
+      option = 0; 
+      if (getsockopt(s, SOL_IP, IP_MTU, &option, &optlen) < 0)
+        writeError(ERR_ERROR, "Error retrieving MTU size.");
+    
+      writeError(ERR_DEBUG, "MTU size: %d", option);
+    
+      #define TCP_MAXSEG  0x02
+      if (getsockopt(s, IPPROTO_TCP, TCP_MAXSEG, &option, &optlen) < 0)
+        writeError(ERR_ERROR, "Error retrieving TCP_MAXSEG size.");
+
+      writeError(ERR_DEBUG, "MSS size: %d", option);
+    */
+
     writeError(ERR_DEBUG, "Connected (internal)");
 
     if (nUseProxy > 0)
@@ -290,7 +320,7 @@ int medusaConnectInternal(unsigned long nHost, int nPort, int nProtocol, int nTy
 }
 
 #ifdef HAVE_LIBSSL
-RSA *sslTempRSACallback(SSL * ssl, int export, int keylength)
+RSA *sslTempRSACallback(SSL * ssl __attribute__((unused)), int export __attribute__((unused)), int keylength __attribute__((unused)))
 {
   /* "rsa" was previously global... do we ever need to return a previously
      generated value? */
@@ -322,12 +352,18 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
     sslContext = SSL_CTX_new(SSLv2_client_method());
   else
 #endif
+#ifndef OPENSSL_NO_SSL3
   if (pParams->nSSLVersion == 3)
     sslContext = SSL_CTX_new(SSLv3_client_method());
-  else if (pParams->nSSLVersion == (float)3.1)
+  else
+#endif
+  if (pParams->nSSLVersion == (float)3.1)
     sslContext = SSL_CTX_new(TLSv1_client_method());
   else
+  {
     sslContext = SSL_CTX_new(SSLv23_client_method());
+    //SSL_CTX_set_options(sslContext, SSL_OP_NO_SSLv3 | SSL_OP_NO_SSLv2);
+  }
 
   if (sslContext == NULL)
   {
@@ -382,7 +418,7 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
 }
 #endif
 
-int medusaReceiveInternal(int socket, char *buf, int length)
+int medusaReceiveInternal(int socket, unsigned char *buf, int length)
 {
 #ifdef HAVE_LIBSSL
   int err;
@@ -429,7 +465,7 @@ int medusaReceiveInternal(int socket, char *buf, int length)
   This is a more robust receive function that can optionally convert NULLS to spaces
   Callers should check the value of *nBufferSize on return - IT MAY HAVE BEEN CHANGED
 */
-char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsToSpaces, int nReceiveDelay1, int nReceiveDelay2)
+unsigned char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsToSpaces, int nReceiveDelay1, int nReceiveDelay2)
 {
   /*
     When receiving UDP packets, we need to be mindful of packets exceeding our default buffer size.
@@ -448,10 +484,15 @@ char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsT
     limit themselves to 512 bytes of user data, to stay below this 576-byte limit. Realistically, 
     however, most implementations today (especially those that support the Network File System, 
     NFS) allow for just over 8192-byte IP datagrams. 
+
+    As of 2013/07/25, we are increasing the buffer size from 576 to 1500. The reason for this
+    change is to deal with a OWA/EWS servers sending non-fragment packets larger than 576 and
+    having data missed. Attempts to negotiate a lower MTU/MSS value with the target Windows
+    host has failed for an unknown reason.
   */
-  const unsigned int BUFFER_SIZE = 576;
+  const int BUFFER_SIZE = 1500;
   
-  char *szBufReceive, *szBufReceiveTmp;
+  unsigned char *szBufReceive, *szBufReceiveTmp;
   int nBufReceive = 0, nBufReceiveTmp = 0, BufReceiveIndex = 0;
   int bSocketStatus = 0;
   int nReceiveDelay1sec = 0, nReceiveDelay1usec = 0;
@@ -532,7 +573,7 @@ char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsT
   return szBufReceive;
 }
 
-int medusaSendInternal(int socket, char *buf, int size, int options)
+int medusaSendInternal(int socket, unsigned char *buf, int size, int options)
 {
 #ifdef HAVE_LIBSSL
   struct SSLSOCKETINFO *s;
@@ -546,19 +587,17 @@ int medusaSendInternal(int socket, char *buf, int size, int options)
   {
 #endif
     int nRet;
-    char *bufReceive = NULL;
-    int nReceiveBufferSize = 0;
+    //char *bufReceive = NULL;
+    //int nReceiveBufferSize = 0;
     
     /* flush any extraneous data remaining on socket */
-    /*
-    while (medusaDataReadyTimed(socket, 0, 1) > 0) 
-    {
-      bufReceive = medusaReceiveRaw(socket, &nReceiveBufferSize);
-      writeError(ERR_DEBUG, "Purging extraneous data on socket from previous request (size %d): %s", nReceiveBufferSize, bufReceive);
-      if (bufReceive == NULL) { break; }
-    }
-    */
-
+    //while (medusaDataReadyTimed(socket, 0, 1) > 0) 
+    //{
+    //  bufReceive = medusaReceiveRaw(socket, &nReceiveBufferSize);
+    //  writeError(ERR_DEBUG, "Purging extraneous data on socket from previous request (size %d): %s", nReceiveBufferSize, bufReceive);
+    //  if (bufReceive == NULL) { break; }
+    //}
+ 
     nRet = send(socket, buf, size, options); 
     if (nRet < 0)
     {
@@ -576,8 +615,8 @@ int medusaSendInternal(int socket, char *buf, int size, int options)
 // Variants of medusaConnectInternal
 int medusaConnect(sConnectParams* pParams)
 {
-  medusaConnectInternal(pParams->nHost, pParams->nPort, pParams->nProtocol, pParams->nType, pParams->nTimeout, pParams->nRetries, pParams->nRetryWait,
-                        pParams->nProxyStringIP, pParams->nProxyStringPort, pParams->szProxyAuthentication, pParams->nSourcePort);
+  return medusaConnectInternal(pParams->nHost, pParams->nPort, pParams->nProtocol, pParams->nType, pParams->nTimeout, pParams->nRetries, pParams->nRetryWait,
+                               pParams->nProxyStringIP, pParams->nProxyStringPort, pParams->szProxyAuthentication, pParams->nSourcePort);
 }
 
 int medusaConnectSSL(sConnectParams* pParams)
@@ -705,7 +744,7 @@ int medusaDataReady(int socket)
 /* Check socket status. Return 1 if connection is still valid. */
 int medusaCheckSocket(int socket, int usec)
 {
-  writeError(ERR_DEBUG, "wait time: %d", usec);
+  writeError(ERR_DEBUG, "Socket Check - wait time: %d", usec);
   
   if (medusaDataReadyTimed(socket, 0, usec) == 0)
     return 1;
@@ -713,7 +752,7 @@ int medusaCheckSocket(int socket, int usec)
     return 0;
 }
 
-int medusaReceive(int socket, char *buf, int length)
+int medusaReceive(int socket, unsigned char *buf, int length)
 {
   int ret;
 
@@ -722,22 +761,22 @@ int medusaReceive(int socket, char *buf, int length)
   return ret;
 }
 
-char* medusaReceiveRaw(int socket, int* nBufferSize)
+unsigned char* medusaReceiveRaw(int socket, int* nBufferSize)
 {
   return medusaReceiveDataInternal(socket, nBufferSize, 0, READ_WAIT_TIME, 0);
 }
 
-char* medusaReceiveRawDelay(int socket, int* nBufferSize, int nReceiveDelay1, int nReceiveDelay2)
+unsigned char* medusaReceiveRawDelay(int socket, int* nBufferSize, int nReceiveDelay1, int nReceiveDelay2)
 {
   return medusaReceiveDataInternal(socket, nBufferSize, 0, nReceiveDelay1, nReceiveDelay2);
 }
 
-char* medusaReceiveLine(int socket, int* nBufferSize)
+unsigned char* medusaReceiveLine(int socket, int* nBufferSize)
 {
   return medusaReceiveDataInternal(socket, nBufferSize, 1, READ_WAIT_TIME, 0);
 }
 
-char* medusaReceiveLineDelay(int socket, int* nBufferSize, int nReceiveDelay1, int nReceiveDelay2)
+unsigned char* medusaReceiveLineDelay(int socket, int* nBufferSize, int nReceiveDelay1, int nReceiveDelay2)
 {
   return medusaReceiveDataInternal(socket, nBufferSize, 1, nReceiveDelay1, nReceiveDelay2);
 }
@@ -754,16 +793,14 @@ char* medusaReceiveLineDelay(int socket, int* nBufferSize, int nReceiveDelay1, i
 */
 int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufReceive, const char* regex)
 {
-  char *szBufReceiveTmp = NULL;
+  unsigned char *szBufReceiveTmp = NULL;
   int nBufReceiveTmp = 0;
   regex_t preg;
   int errcode = REG_NOMATCH;
   char errmsg[512];
-  const char *error = NULL;
-  int erroffset = 0;
   int nAttempt = 1;
-  const unsigned int BUFFER_SIZE = 576;
-
+  const int BUFFER_SIZE = 1500;
+    
   writeError(ERR_DEBUG, "Regural expession: \"%s\"", regex);
   errcode = regcomp(&preg, regex, REG_EXTENDED|REG_ICASE|REG_NOSUB);
   if (errcode)
@@ -780,7 +817,7 @@ int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufRecei
 
   do
   {
-    errcode = regexec(&preg, *szBufReceive, 0, 0, 0);
+    errcode = regexec(&preg, (char* const) *szBufReceive, 0, 0, 0);
     if (errcode == REG_NOMATCH)
     {
       writeError(ERR_DEBUG, "Failed to match regex. Checking for additional data.");
@@ -831,7 +868,7 @@ int medusaReceiveRegex(int hSocket, unsigned char **szBufReceive, int* nBufRecei
   return FAILURE;
 }
 
-int medusaSend(int socket, char *buf, int size, int options)
+int medusaSend(int socket, unsigned char *buf, int size, int options)
 {
   char debugbuf[size + 1];
   int k;
