@@ -23,7 +23,7 @@
  *
 */
 
-#define VERSION_SVN "$Id: medusa.c 1666 2012-02-09 19:40:02Z jmk $" 
+#define VERSION_SVN "$Id: medusa.c 9217 2015-05-07 18:07:03Z jmk $" 
 
 #include <dlfcn.h>
 #include "medusa.h"
@@ -334,7 +334,7 @@ int checkOptions(int argc, char **argv, sAudit *_psAudit)
   
   if (_psAudit->iShowModuleHelp)
   {
-    ret = invokeModule(_psAudit->pModuleName, NULL, NULL, NULL);
+    ret = invokeModule(_psAudit->pModuleName, NULL, 0, NULL);
     if (ret < 0)
     {
       writeError(ERR_CRITICAL, "invokeModule failed - see previous errors for an explanation");
@@ -912,8 +912,6 @@ char* findLocalPass(sAudit *_psAudit)
 
 int loadLoginInfo(sAudit *_psAudit)
 {
-  int res;
-
   sHost *psHost = NULL;
   sHost *psHostPrevTmp = NULL;
   char *pHost = NULL;
@@ -1370,7 +1368,7 @@ int getNextCredSet(sLogin *_psLogin, sCredentialSet *_psCredSet)
 /*
   Process password result from login module
 */
-int setPassResult(sLogin *_psLogin, char *_pPass)
+void setPassResult(sLogin *_psLogin, char *_pPass)
 {
   pthread_mutex_lock(&_psLogin->psServer->ptmMutex);
 
@@ -1497,31 +1495,28 @@ int addMissedCredSet(sLogin *_psLogin, sCredentialSet *_psCredSet)
   _psLogin->psServer->iCredentialsMissed++;
 
   pthread_mutex_unlock(&_psLogin->psServer->ptmMutex);
+
+  return SUCCESS;
 }
 
 
-void* startModule(void* pParams)
+void startModule(void* pParams)
 {
   int64_t nRet = 0;
   sModuleStart* modParams = (sModuleStart*)pParams;
   if (NULL == modParams)
   {
     writeError(ERR_FATAL, "Bad pointer passed to invokeModule");
-    nRet = -1;
-    return (void*) nRet;
+    return;
   }
 
   writeError(ERR_DEBUG, "startModule iId: %d pLogin: %X modParams->argv: %X modParams: %X", modParams->pLogin->iId, modParams->pLogin, modParams->argv, modParams);
   
   nRet = invokeModule(modParams->szModuleName, modParams->pLogin, modParams->argc, modParams->argv);
   if (nRet < 0)
-  {
     writeVerbose(VB_EXIT, "invokeModule failed - see previous errors for an explanation");
-    nRet = -1;
-    return (void*) nRet;
-  }
 
-  return (void*) nRet;    // Success
+  return;
 }
 
 
@@ -1530,7 +1525,7 @@ void* startModule(void* pParams)
   has a single thread for this purpose. The thread spawns multiple child 
   threads which each initiate the selected module to perform the actual logons.
 */
-void *startLoginThreadPool(void *arg)
+void startLoginThreadPool(void *arg)
 {
   sServer *_psServer = (sServer *)arg;
   thr_pool_t *login_pool = NULL;
@@ -1744,7 +1739,7 @@ int startServerThreadPool(sAudit *_psAudit)
       {    
         writeError(ERR_DEBUG_SERVER, "[Host Resume] Adding host: %d (we've passed the point of the previous run)", psHost->iId);
       }
-      else if (szResumeMap = strstr(_psAudit->pOptResume, szTmp1))
+      else if ((szResumeMap = strstr(_psAudit->pOptResume, szTmp1)))
       {
         writeError(ERR_DEBUG_SERVER, "[Host Resume] Adding host: %d (host was located in resume map)", psHost->iId);
 
@@ -1824,7 +1819,6 @@ int startServerThreadPool(sAudit *_psAudit)
       psServer[iServerId].iLoginsDone = 0;
       psServer[iServerId].iCredentialsMissed = 0;
     
-      //if ( thr_pool_queue(server_pool, startLoginThreadPool, (void *)psServer + iServerId * sizeof(sServer)) < 0 )
       if ( thr_pool_queue(_psAudit->server_pool, startLoginThreadPool, (void *) &psServer[iServerId]) < 0 )
       {
         writeError(ERR_ERROR, "Failed to add host task to server thread pool.");
@@ -1860,7 +1854,7 @@ int startServerThreadPool(sAudit *_psAudit)
   which were partially tested will be resumed from the start of their password
   list.
 */
-void sigint_handler(int sig)
+void sigint_handler(int sig __attribute__((unused)))
 {
   sHost *psHost;
   sUser *psUser;
@@ -1980,10 +1974,10 @@ void sigint_handler(int sig)
   exit(0);
 }
 
-int main(int argc, char **argv, char *envp[])
+int main(int argc, char **argv, char *envp[] __attribute__((unused)))
 {
   struct sigaction sig_action;
-  int iExitStatus;
+  int iExitStatus = EXIT_SUCCESS;
   int i;
 
   struct tm *tm_ptr;
