@@ -270,5 +270,12 @@ typedef struct __sModuleStart
   char**  argv;  
 } sModuleStart;
 
-#define _MEDUSA_H
+
+void listModules(char* arrPaths[], int nTerminateNow);
+int invokeModule(char* pModuleName, sLogin* pLogin, int argc, char* argv[]);
+
+int getNextCredSet(sLogin *_psLogin, sCredentialSet *_psCredSet);
+void setPassResult(sLogin *_psLogin, char *_pPass);
+int addMissedCredSet(sLogin *_psLogin, sCredentialSet *_psCredSet);
+
 #endif
