@@ -7,6 +7,12 @@
   sample code.
 */
 
+/*
+#pragma GCC diagnostic push
+#pragma GCC diagnostic warning "-Werror"
+#pragma GCC diagnostic push
+#pragma GCC diagnostic warning "-Wall"
+*/
 
 #include <string.h>
 #include "http-digest.h"
@@ -117,3 +123,8 @@ void DigestCalcResponse(
       MD5_Final(RespHash, &Md5Ctx);
       CvtHex(RespHash, Response);
 };
+
+/*
+#pragma GCC diagnostic pop
+#pragma GCC diagnostic pop
+*/
