@@ -6,9 +6,9 @@
 #include <openssl/md5.h>
 
 #define HASHLEN 16
-typedef char HASH[HASHLEN];
+typedef unsigned char HASH[HASHLEN];
 #define HASHHEXLEN 32
-typedef char HASHHEX[HASHHEXLEN+1];
+typedef unsigned char HASHHEX[HASHHEXLEN+1];
 #define IN
 #define OUT
 
