@@ -47,7 +47,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for M$-SQL sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: mssql.c 1719 2012-05-23 16:56:04Z jmk $"
+#define MODULE_VERSION_SVN "$Id: mssql.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -76,7 +76,7 @@ enum MODULE_STATE
 
 // Forward declarations
 int connectMSSQL(sLogin *psLogin, _MODULE_DATA *_psSessionData);
-int tryLogin(int hSocket, sLogin** login, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword);
+int tryLogin(int hSocket, sLogin** login, char* szLogin, char* szPassword);
 int initModule(sLogin* login, _MODULE_DATA* _psSessionData);
 
 // Tell medusa how many parameters this module allows
@@ -113,21 +113,18 @@ void showUsage()
 }
 
 // The "main" of the medusa module world - this is what gets called to actually do the work
-int go(sLogin* logins, int argc, char *argv[])
+int go(sLogin* logins, int argc, char *argv[] __attribute__((unused)))
 {
-  int i;
-
-  char *strtok_ptr, *pOpt, *pOptTmp;
   _MODULE_DATA *psSessionData;
 
   psSessionData = malloc(sizeof(_MODULE_DATA));
   memset(psSessionData, 0, sizeof(_MODULE_DATA));
 
-  if ( !(0 <= argc <= 3) )
+  if (argc != 0)
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
-  } 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
+  }
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
@@ -143,7 +140,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive = NULL;
   sCredentialSet *psCredSet = NULL;
 
   _psSessionData->nPort = 0;
@@ -181,7 +177,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+        nState = tryLogin(hSocket, &psLogin, psCredSet->psUser->pUser, psCredSet->pPass);
         
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
         {
@@ -244,7 +240,7 @@ int connectMSSQL(sLogin *_psLogin, _MODULE_DATA *_psSessionData)
 {
   sConnectParams params;
   int nReceiveBufferSize = 0;
-  char *bufReceive = NULL; 
+  unsigned char *bufReceive = NULL; 
   int nPort = PORT_MSSQL; 
   int nPortTmp = 0; 
   int hSocket;
@@ -306,7 +302,7 @@ int connectMSSQL(sLogin *_psLogin, _MODULE_DATA *_psSessionData)
     bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
     if (bufReceive == NULL)
     {
-      writeError(ERR_ERROR, "[%s] SQL server (%s) did not response to port query request. Using default value of 1433/tcp.", MODULE_NAME, _psLogin->psServer->pHostIP);
+      writeError(ERR_ERROR, "[%s] SQL server (%s) did not respond to port query request. Using default value of 1433/tcp.", MODULE_NAME, _psLogin->psServer->pHostIP);
       nPort = 1433;
     } 
     else if (bufReceive[0] == 0x05)
@@ -320,8 +316,8 @@ int connectMSSQL(sLogin *_psLogin, _MODULE_DATA *_psSessionData)
         writeError(ERR_ERROR, "[%s] Reported SQL ping data length does not match our receive buffer length. Response data may have been lost.", MODULE_NAME);
 
       nSQLInstance = 1;
-      szTmp = bufReceive + 3;
-      while (szTmp = strstr(szTmp, "ServerName;"))
+      szTmp = (char*)bufReceive + 3;
+      while ((szTmp = strstr(szTmp, "ServerName;")) != NULL)
       {
         szTmp1 = strstr(szTmp, ";;");
         
@@ -483,7 +479,7 @@ void makeSQLLogin(char* szLogin, char* szPassword, unsigned char* buffer)
   return;
 }
 
-int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
+int tryLogin(int hSocket, sLogin** psLogin, char* szLogin, char* szPassword)
 {
   int iRet;
   unsigned char bufSend[3 * MSLEN + 422 + 1];
