@@ -40,7 +40,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for NCP sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: ncp.c 1680 2012-02-29 19:12:33Z jmk $"
+#define MODULE_VERSION_SVN "$Id: ncp.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define LIBNCP_WARNING "No usable LIBNCP. Module disabled."
@@ -65,7 +65,7 @@ enum MODULE_STATE
 };
 
 // Forward declarations
-int tryLogin(int hSocket, sLogin** login, _NCP_DATA* _psSessionData, char* szLogin, char* szPassword);
+int tryLogin(sLogin** login, _NCP_DATA* _psSessionData, char* szPassword);
 int initModule(sLogin* login, _NCP_DATA *_psSessionData);
 
 // Tell medusa how many parameters this module allows
@@ -132,11 +132,11 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_NCP_DATA));  
   memset(psSessionData, 0, sizeof(_NCP_DATA));
 
-  if ( !(0 <= argc <= 3) )
+  if ((argc < 0) || (argc > 1))
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
-  } 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
+  }
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
@@ -248,7 +248,7 @@ int initModule(sLogin* psLogin, _NCP_DATA *_psSessionData)
         nState = MSTATE_RUNNING;
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(0, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+        nState = tryLogin(&psLogin, _psSessionData, psCredSet->pPass);
 
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
         {
@@ -292,9 +292,10 @@ int initModule(sLogin* psLogin, _NCP_DATA *_psSessionData)
   return SUCCESS;
 }
 
-int tryLogin(int hSocket, sLogin** psLogin, _NCP_DATA* _psSessionData, char* szLogin, char* szPassword)
+int tryLogin(sLogin** psLogin, _NCP_DATA* _psSessionData, char* szPassword)
 {
-  int i, iRet;
+  unsigned int i;
+  int iRet;
   short int NCPErrorCode = 0;
   char *pErrorMsg = NULL;
   char ErrorCode[12];
