@@ -38,7 +38,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for NNTP sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: nntp.c 1301 2010-02-09 22:54:18Z jmk $"
+#define MODULE_VERSION_SVN "$Id: nntp.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300 
@@ -61,9 +61,9 @@ enum MODULE_STATE
 
 // Forward declarations
 int initConnection(int hSocket, sLogin** psLogin);
-int terminateConnection(int hSocket, sLogin** psLogin);
-int tryLogin(int hSocket, sLogin** login, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword);
-int initModule(sLogin* login, _MODULE_DATA *_psSessionData);
+int terminateConnection(int hSocket);
+int tryLogin(int hSocket, sLogin** login, char* szLogin, char* szPassword);
+int initModule(sLogin* login);
 
 // Tell medusa how many parameters this module allows
 int getParamNumber()
@@ -100,29 +100,29 @@ void showUsage()
 }
 
 // The "main" of the medusa module world - this is what gets called to actually do the work
-int go(sLogin* logins, int argc, char *argv[])
+int go(sLogin* logins, int argc, char *argv[] __attribute__((unused)))
 {
   _MODULE_DATA *psSessionData;
 
   psSessionData = malloc(sizeof(_MODULE_DATA));
   memset(psSessionData, 0, sizeof(_MODULE_DATA));
 
-  if ( !(0 <= argc <= 3) )
-  {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
+  if (argc != 0)
+  { 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
   } 
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
-    initModule(logins, psSessionData);
+    initModule(logins);
   }  
 
   FREE(psSessionData);
   return SUCCESS;
 }
 
-int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
+int initModule(sLogin* psLogin)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
@@ -181,7 +181,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
  
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+        nState = tryLogin(hSocket, &psLogin, psCredSet->psUser->pUser, psCredSet->pPass);
 
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
         {
@@ -209,7 +209,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         break;
       case MSTATE_EXITING:
         if (hSocket > 0)
-          terminateConnection(hSocket, &psLogin);
+          terminateConnection(hSocket);
         hSocket = -1;
         nState = MSTATE_COMPLETE;
         break;
@@ -228,14 +228,14 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 }
 
 /* Module Specific Functions */
-int terminateConnection(int hSocket, sLogin** psLogin)
+int terminateConnection(int hSocket)
 {
   unsigned char bufSend[BUF_SIZE];
 
   memset(bufSend, 0, sizeof(bufSend));
-  sprintf(bufSend, "QUIT\r\n");
+  sprintf((char*)bufSend, "QUIT\r\n");
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -260,12 +260,12 @@ int initConnection(int hSocket, sLogin** psLogin)
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
     return MSTATE_EXITING;
   }
-  else if ((strstr(bufReceive,"200") != NULL)) 
+  else if ((strstr((char*)bufReceive,"200") != NULL)) 
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Retrieved NNTP service banner.", MODULE_NAME);
   }
   /* 400 Too Many Connections */
-  else if ((strstr(bufReceive,"400") != NULL)) 
+  else if ((strstr((char*)bufReceive,"400") != NULL)) 
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Too many connections from host recieved by server.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
@@ -280,9 +280,9 @@ int initConnection(int hSocket, sLogin** psLogin)
      
   /* Verify that authentication is required */ 
   memset(bufSend, 0, sizeof(bufSend));
-  sprintf(bufSend, "HELP\r\n");
+  sprintf((char*)bufSend, "HELP\r\n");
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_UNKNOWN;
@@ -298,7 +298,7 @@ int initConnection(int hSocket, sLogin** psLogin)
     iRet = MSTATE_EXITING;
   }
   /* 480 Authentication Required */ 
-  else if (strstr(bufReceive,"480") != NULL) 
+  else if (strstr((char*)bufReceive, "480") != NULL) 
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Server requires authentication.", MODULE_NAME);
     iRet = MSTATE_RUNNING;
@@ -314,7 +314,7 @@ int initConnection(int hSocket, sLogin** psLogin)
 }
 
 
-int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
+int tryLogin(int hSocket, sLogin** psLogin, char* szLogin, char* szPassword)
 {
   int iRet;
   unsigned char bufSend[BUF_SIZE];
@@ -323,9 +323,9 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
 
   /* send username */
   memset(bufSend, 0, sizeof(bufSend));
-  sprintf(bufSend, "AUTHINFO USER %.250s\r\n", szLogin);
+  sprintf((char*)bufSend, "AUTHINFO USER %.250s\r\n", szLogin);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -338,7 +338,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     return FAILURE;
   }
   /* 381 More Authentication Required */
-  else if (strstr(bufReceive,"381") != NULL) 
+  else if (strstr((char*)bufReceive,"381") != NULL) 
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Receive 381 response requesting user password.", MODULE_NAME);
   }
@@ -352,12 +352,12 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   /* send password */
   memset(bufSend, 0, sizeof(bufSend));
   
-  if (strlen(szPassword) == 0) 
-    sprintf(bufSend, "AUTHINFO PASS \"\"\r\n");
+  if (strlen((char*)szPassword) == 0) 
+    sprintf((char*)bufSend, "AUTHINFO PASS \"\"\r\n");
   else
-    sprintf(bufSend, "AUTHINFO PASS %.250s\r\n", szPassword);
+    sprintf((char*)bufSend, "AUTHINFO PASS %.250s\r\n", szPassword);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -370,28 +370,28 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     return FAILURE;
   }
   /* 281 Authentication accepted */
-  else if (strstr(bufReceive,"281") != NULL)
+  else if (strstr((char*)bufReceive, "281") != NULL)
   {
     writeError(ERR_ERROR, "[%s] Login attempt successful.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
     iRet = MSTATE_EXITING;
   }
   /* 482 Authentication rejected */
-  else if (strstr(bufReceive,"482") != NULL)
+  else if (strstr((char*)bufReceive, "482") != NULL)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt rejected.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
     iRet = MSTATE_RUNNING;
   }
   /* 501 Bad Command Usage */
-  else if (strstr(bufReceive,"501") != NULL)
+  else if (strstr((char*)bufReceive, "501") != NULL)
   {
     writeError(ERR_ERROR, "[%s] Bad command usage.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
     iRet = MSTATE_RUNNING;
   }
   /* 502 Authentication Failed */
-  else if (strstr(bufReceive,"502") != NULL)
+  else if (strstr((char*)bufReceive, "502") != NULL)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
