@@ -36,7 +36,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for POP3 sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: pop3.c 1722 2012-05-23 16:57:27Z jmk $"
+#define MODULE_VERSION_SVN "$Id: pop3.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 
 #define BUF_SIZE 300 
@@ -130,11 +130,11 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_MODULE_DATA));
   memset(psSessionData, 0, sizeof(_MODULE_DATA));
 
-  if ( !(0 <= argc <= 3) )
+  if ((argc < 0) || (argc > 3))
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
-  } 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
+  }
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
@@ -289,8 +289,8 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
         if (psLogin->psServer->psHost->iUseSSL == 0)
         {
           memset(bufSend, 0, BUF_SIZE);
-          sprintf(bufSend, "STLS\r\n");
-          if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+          sprintf((char*)bufSend, "STLS\r\n");
+          if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
           {
             writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
             return FAILURE;
@@ -307,7 +307,7 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
             [NOT SUPPORTED] +OK STLS completed
             [ERROR]         -ERR Command not permitted when TLS active
           */
-          else if (strstr(bufReceive, "+OK") != NULL)
+          else if (strstr((char*)bufReceive, "+OK") != NULL)
           {
             FREE(bufReceive);
   
@@ -407,14 +407,13 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
 {
   unsigned char* bufReceive;
   unsigned char* bufSend;
-  int nSendBufferSize = 0;
   int nReceiveBufferSize = 0;
 
   bufSend = malloc(6 + 1);
   memset(bufSend, 0, 6 + 1);
-  sprintf(bufSend, "CAPA\r\n");
+  sprintf((char*)bufSend, "CAPA\r\n");
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
     return FAILURE;
@@ -426,24 +425,24 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
     writeError(ERR_ERROR, "[%s] Failed: Server did not respond that it supported any of the authentication types we handle (USER, LOGIN, and NTLM). Use the AUTH module option to force the use of an authentication type: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
-  else if ((strstr(bufReceive,"USER") != NULL))
+  else if ((strstr((char*)bufReceive, "USER") != NULL))
   {
     writeError(ERR_DEBUG_MODULE, "Server requested authentication type: USER (clear-text)");
     _psSessionData->nAuthType = AUTH_USER;
   }
-  else if ((strstr(bufReceive,"SASL") != NULL))
+  else if ((strstr((char*)bufReceive, "SASL") != NULL))
   {
-    if ((strstr(bufReceive,"PLAIN") != NULL))
+    if ((strstr((char*)bufReceive, "PLAIN") != NULL))
     {
       writeError(ERR_DEBUG_MODULE, "Server requested authentication type: SASL PLAIN");
       _psSessionData->nAuthType = AUTH_PLAIN;
     }
-    else if ((strstr(bufReceive,"LOGIN") != NULL))
+    else if ((strstr((char*)bufReceive, "LOGIN") != NULL))
     {
       writeError(ERR_DEBUG_MODULE, "Server requested authentication type: SASL LOGIN");
       _psSessionData->nAuthType = AUTH_LOGIN;
     }
-    else if ((strstr(bufReceive,"NTLM") != NULL))
+    else if ((strstr((char*)bufReceive, "NTLM") != NULL))
     {
       writeError(ERR_DEBUG_MODULE, "Server requested authentication type: SASL NTLM");
       _psSessionData->nAuthType = AUTH_NTLM;
@@ -454,7 +453,7 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData)
       return FAILURE;
     }
   }
-  else if ((strstr(bufReceive,"-ERR") != NULL))
+  else if ((strstr((char*)bufReceive, "-ERR") != NULL))
   {
     writeError(ERR_ERROR, "[%s] Server did not understand CAPA request. Defaulting to USER authentication type, use \"-m AUTH\" option to specify alternative method.", MODULE_NAME);
     _psSessionData->nAuthType = AUTH_USER;
@@ -476,11 +475,11 @@ int sendAuthUSER(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   memset(bufSend, 0, sizeof(bufSend));
 
   if (_psSessionData->szDomain)
-    sprintf(bufSend, "USER %.100s@%.150s\r\n", szLogin, _psSessionData->szDomain);
+    sprintf((char*)bufSend, "USER %.100s@%.150s\r\n", szLogin, _psSessionData->szDomain);
   else
-    sprintf(bufSend, "USER %.250s\r\n", szLogin);
+    sprintf((char*)bufSend, "USER %.250s\r\n", szLogin);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -491,18 +490,18 @@ int sendAuthUSER(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     writeError(ERR_ERROR, "[%s] Failed: Server did not respond as expected to USER authentication attempt: %s", MODULE_NAME, bufReceive);
     return FAILURE;
   }
-  else if (strstr(bufReceive, " signing off."))
+  else if (strstr((char*)bufReceive, " signing off."))
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Server informed us it was signing off. Restarting connection.", MODULE_NAME);
     nRet = MSTATE_NEW;
     return(nRet);
   }
-  else if (strstr(bufReceive, "ERR Cleartext login on this server requires the use of transport level security (SSL/TLS)"))
+  else if (strstr((char*)bufReceive, "ERR Cleartext login on this server requires the use of transport level security (SSL/TLS)"))
   {
     writeError(ERR_ERROR, "[%s] Server requires use of SSL/TLS.", MODULE_NAME);
     return FAILURE;
   }
-  else if (strstr(bufReceive, "ERR Clear text passwords have been disabled for this protocol."))
+  else if (strstr((char*)bufReceive, "ERR Clear text passwords have been disabled for this protocol."))
   {
     writeError(ERR_ERROR, "[%s] Server does not accept clear-text password authentication.", MODULE_NAME);
     return FAILURE;
@@ -510,9 +509,9 @@ int sendAuthUSER(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
  
   /* send password */
   memset(bufSend, 0, sizeof(bufSend));
-  sprintf(bufSend, "PASS %.250s\r\n", szPassword);
+  sprintf((char*)bufSend, "PASS %.250s\r\n", szPassword);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -528,15 +527,12 @@ int sendAuthUSER(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   Example:
     AUTH PLAIN dGVzdAB0ZXN0AHRlc3Q=
 */
-int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
+int sendAuthPLAIN(int hSocket, char* szLogin, char* szPassword)
 {
-  unsigned char* bufReceive = NULL;
   unsigned char* bufSend = NULL;
-  unsigned char* szTmpBuf = NULL;
-  unsigned char* szTmpBuf64 = NULL;
+  char* szTmpBuf = NULL;
+  char* szTmpBuf64 = NULL;
   int nSendBufferSize = 0;
-  int nReceiveBufferSize = 0;
-  int nRet = SUCCESS;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Initiating PLAIN Authentication Attempt.", MODULE_NAME);
 
@@ -556,10 +552,10 @@ int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   bufSend = malloc(11 + strlen(szTmpBuf64) + 2 + 1); 
   memset(bufSend, 0, 11 + strlen(szTmpBuf64) + 2 + 1);
 
-  sprintf(bufSend, "AUTH PLAIN %s\r\n", szTmpBuf64);
+  sprintf((char*)bufSend, "AUTH PLAIN %s\r\n", szTmpBuf64);
   FREE(szTmpBuf64);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -577,24 +573,22 @@ int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
       + UGFzc3dvcmQ6      (Password:)
       YmFy                (bar)
 */
-int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
+int sendAuthLOGIN(int hSocket, char* szLogin, char* szPassword)
 {
   unsigned char* bufReceive = NULL;
   unsigned char* bufSend = NULL;
-  unsigned char* szPrompt = NULL;
-  unsigned char* szTmpBuf = NULL;
-  int nSendBufferSize = 0;
+  char* szPrompt = NULL;
+  char* szTmpBuf = NULL;
   int nReceiveBufferSize = 0;
-  int nRet = SUCCESS;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Initiating LOGIN Authentication Attempt.", MODULE_NAME);
 
   /* --- Send initial AUTH LOGIN command --- */
   bufSend = malloc(12 + 1);
   memset(bufSend, 0, 12 + 1);
-  sprintf(bufSend, "AUTH LOGIN\r\n");
+  sprintf((char*)bufSend, "AUTH LOGIN\r\n");
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -607,18 +601,18 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
     writeError(ERR_ERROR, "[%s] POP3 server did not respond with \"+ \" to AUTH LOGIN request.", MODULE_NAME);
     return FAILURE;
   }
-  else if (strstr(bufReceive,"-ERR The specified authentication package is not supported.") != NULL) 
+  else if (strstr((char*)bufReceive,"-ERR The specified authentication package is not supported.") != NULL) 
   {
     writeError(ERR_ERROR, "[%s] Server response: The specified authentication package is not supported.", MODULE_NAME);
     return FAILURE;
   }
 
-  szTmpBuf = ((char*)index(bufReceive, '\r'));
+  szTmpBuf = index((char*)bufReceive, '\r');
   szTmpBuf[0] = '\0';
-  szPrompt = malloc(strlen(bufReceive + 2) + 1);
-  memset(szPrompt, 0, strlen(bufReceive + 2) + 1);
+  szPrompt = malloc(strlen((char*)bufReceive + 2) + 1);
+  memset(szPrompt, 0, strlen((char*)bufReceive + 2) + 1);
   
-  base64_decode(bufReceive + 2, szPrompt);
+  base64_decode((char*)bufReceive + 2, szPrompt);
   FREE(bufReceive);
 
   writeError(ERR_DEBUG_MODULE, "[%s] POP3 server sent the following prompt: %s", MODULE_NAME, szPrompt); 
@@ -629,10 +623,10 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   /* Base64 encoded value can be up to 2x+2 original text. Leave additional space for "\r\n" and NULL */
   bufSend = malloc((2 * strlen(szLogin) + 2) + 2 + 1);
   memset(bufSend, 0, (2 * strlen(szLogin) + 2) + 2 + 1);
-  base64_encode(szLogin, strlen(szLogin), bufSend);
-  strncat(bufSend, "\r\n", 2);   
+  base64_encode(szLogin, strlen(szLogin), (char*)bufSend);
+  strncat((char*)bufSend, "\r\n", 2);   
  
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -645,12 +639,12 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
     return FAILURE;
   }
 
-  szTmpBuf = ((char*)index(bufReceive, '\r'));
+  szTmpBuf = index((char*)bufReceive, '\r');
   szTmpBuf[0] = '\0';
-  szPrompt = malloc(strlen(bufReceive + 2) + 1);
-  memset(szPrompt, 0, strlen(bufReceive + 2) + 1);
+  szPrompt = malloc(strlen((char*)bufReceive + 2) + 1);
+  memset(szPrompt, 0, strlen((char*)bufReceive + 2) + 1);
   
-  base64_decode(bufReceive + 2, szPrompt);
+  base64_decode((char*)bufReceive + 2, szPrompt);
   FREE(bufReceive);
 
   writeError(ERR_DEBUG_MODULE, "[%s] POP3 server sent the following prompt: %s", MODULE_NAME, szPrompt); 
@@ -661,10 +655,10 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   /* Base64 encoded value can be up to 2x+2 original text. Leave additional space for "\r\n" and NULL */
   bufSend = malloc((2 * strlen(szPassword) + 2) + 2 + 1);
   memset(bufSend, 0, (2 * strlen(szPassword) + 2) + 2 + 1);
-  base64_encode(szPassword, strlen(szPassword), bufSend);
-  strncat(bufSend, "\r\n", 2);   
+  base64_encode(szPassword, strlen(szPassword), (char*)bufSend);
+  strncat((char*)bufSend, "\r\n", 2);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -687,17 +681,17 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   tSmbNtlmAuthRequest   sTmpReq;
   tSmbNtlmAuthChallenge sTmpChall;
   tSmbNtlmAuthResponse  sTmpResp;
-  unsigned char* szTmpBuf = NULL;
-  unsigned char* szTmpBuf64 = NULL;
+  char* szTmpBuf = NULL;
+  char* szTmpBuf64 = NULL;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Initiating NTLM Authentication Attempt.", MODULE_NAME);
 
   /* --- Send initial AUTHENTICATE NTLM command --- */
   bufSend = malloc(11 + 1);
   memset(bufSend, 0, 11 + 1);
-  sprintf(bufSend, "AUTH NTLM\r\n");
+  sprintf((char*)bufSend, "AUTH NTLM\r\n");
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -724,11 +718,11 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
-  sprintf(bufSend, "%s\r\n", szTmpBuf64);
+  sprintf((char*)bufSend, "%s\r\n", szTmpBuf64);
 
   FREE(szTmpBuf64);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char*)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
     return FAILURE;
@@ -744,11 +738,11 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     return FAILURE;
   }
 
-  szTmpBuf = ((char*)index(bufReceive, '\r'));
+  szTmpBuf = index((char*)bufReceive, '\r');
   szTmpBuf[0] = '\0';
 
   writeError(ERR_DEBUG_MODULE, "[%s] NTLM Challenge (B64 Encoded): %s", MODULE_NAME, bufReceive + 2);
-  base64_decode(bufReceive + 2, (char *)&sTmpChall);
+  base64_decode((char*)bufReceive + 2, (char *)&sTmpChall);
 
   FREE(bufReceive);
 
@@ -764,7 +758,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   nSendBufferSize = strlen(szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
-  sprintf(bufSend, "%s\r\n", szTmpBuf64);
+  sprintf((char*)bufSend, "%s\r\n", szTmpBuf64);
 
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
@@ -795,11 +789,11 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
       break;
     case AUTH_PLAIN:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending PLAIN Authentication.", MODULE_NAME);
-      nRet = sendAuthPLAIN(hSocket, _psSessionData, szLogin, szPassword);
+      nRet = sendAuthPLAIN(hSocket, szLogin, szPassword);
       break;
     case AUTH_LOGIN:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending LOGIN Authentication.", MODULE_NAME);
-      nRet = sendAuthLOGIN(hSocket, _psSessionData, szLogin, szPassword);
+      nRet = sendAuthLOGIN(hSocket, szLogin, szPassword);
       break;
     case AUTH_NTLM:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending NTLM Authentication.", MODULE_NAME);
@@ -848,7 +842,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
     nRet = MSTATE_EXITING;
   }
-  else if (strstr(bufReceive,"-ERR The specified authentication package is not supported.") != NULL) 
+  else if (strstr((char*)bufReceive, "-ERR The specified authentication package is not supported.") != NULL) 
   {
     writeError(ERR_ERROR, "[%s] Server response: The specified authentication package is not supported.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_ERROR;
@@ -862,37 +856,37 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
         www.venera.com/downloads/Enumeration_of_AS400_users_via_pop3.pdf 
         Example: -ERR Logon attempt invalid CPF2204
       */
-      if (strstr(bufReceive, "CPF2204"))
+      if (strstr((char*)bufReceive, "CPF2204"))
       {
         writeError(ERR_ERROR, "[%s] User profile was not found.", MODULE_NAME);
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
         nRet = MSTATE_EXITING;
       }
-      else if (strstr(bufReceive, "CPF22E2"))
+      else if (strstr((char*)bufReceive, "CPF22E2"))
       {
         writeError(ERR_DEBUG_MODULE, "[%s] Valid user, incorrect password.", MODULE_NAME);
         (*psLogin)->iResult = LOGIN_RESULT_FAIL;
         nRet = MSTATE_NEW;
       }
-      else if (strstr(bufReceive, "CPF22E3"))
+      else if (strstr((char*)bufReceive, "CPF22E3"))
       {
         writeError(ERR_ERROR, "[%s] Valid user, but profile is disabled.", MODULE_NAME);
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
         nRet = MSTATE_EXITING;
       }
-      else if (strstr(bufReceive, "CPF22E4"))
+      else if (strstr((char*)bufReceive, "CPF22E4"))
       {
         writeError(ERR_ERROR, "[%s] Valid user, but password for profile has expired.", MODULE_NAME);
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
         nRet = MSTATE_EXITING;
       }
-      else if (strstr(bufReceive, "CPF22E5"))
+      else if (strstr((char*)bufReceive, "CPF22E5"))
       {
         writeError(ERR_ERROR, "[%s] Valid user, but no password associated with user profile.", MODULE_NAME);
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
         nRet = MSTATE_EXITING;
       }
-      else if (strstr(bufReceive, "-ERR Logon attempt invalid"))
+      else if (strstr((char*)bufReceive, "-ERR Logon attempt invalid"))
       {
         writeError(ERR_DEBUG_MODULE, "[%s] Generic POP3 invalid attempt message. AS/400 may not be configured for verbose messages.", MODULE_NAME);
         (*psLogin)->iResult = LOGIN_RESULT_FAIL;
