@@ -35,7 +35,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for PostgreSQL sessions"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: postgres.c 1301 2010-02-09 22:54:18Z jmk $"
+#define MODULE_VERSION_SVN "$Id: postgres.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define LIBPQ_WARNING "No usable LIBPQ. Module disabled."
@@ -61,7 +61,7 @@ enum MODULE_STATE
 };
 
 // Forward declarations
-int tryLogin(int hSocket, sLogin** login, _POSTGRESQL_DATA* _psSessionData, char* szLogin, char* szPassword);
+int tryLogin(sLogin** login, _POSTGRESQL_DATA* _psSessionData, char* szLogin, char* szPassword);
 int initModule(sLogin* login, _POSTGRESQL_DATA *_psSessionData);
 
 // Tell medusa how many parameters this module allows
@@ -111,11 +111,11 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_POSTGRESQL_DATA));
   memset(psSessionData, 0, sizeof(_POSTGRESQL_DATA));
 
-  if ( !(0 <= argc <= 3) )
+  if ((argc < 0) || (argc > 1))
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
-  } 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
+  }
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
@@ -219,7 +219,7 @@ int initModule(sLogin* psLogin, _POSTGRESQL_DATA *_psSessionData)
 
         break;
       case MSTATE_RUNNING:
-        nState = tryLogin(hSocket, &psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
+        nState = tryLogin(&psLogin, _psSessionData, psCredSet->psUser->pUser, psCredSet->pPass);
 
         if (psLogin->iResult != LOGIN_RESULT_UNKNOWN)
         {
@@ -267,7 +267,7 @@ int initModule(sLogin* psLogin, _POSTGRESQL_DATA *_psSessionData)
 
 /* Module Specific Functions */
 
-int tryLogin(int hSocket, sLogin** psLogin, _POSTGRESQL_DATA* _psSessionData, char* szLogin, char* szPassword)
+int tryLogin(sLogin** psLogin, _POSTGRESQL_DATA* _psSessionData, char* szLogin, char* szPassword)
 {
   char *szConnectionString;
   int iRet;
