@@ -55,3 +55,6 @@ typedef struct SHA1_CONTEXT
   uint8 Message_Block[64];	/* 512-bit message blocks      */
 } SHA1_CONTEXT;
 
+int sha1_reset(SHA1_CONTEXT *context);
+int sha1_input(SHA1_CONTEXT *context, const uint8 *message_array, unsigned length);
+int sha1_result(SHA1_CONTEXT *context, uint8 Message_Digest[SHA1_HASH_SIZE]);
