@@ -91,8 +91,8 @@
 #define MODULE_NAME    "smbnt.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SMB (LM/NTLM/LMv2/NTLMv2) sessions"
-#define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: smbnt.c 1682 2012-02-29 19:13:03Z jmk $"
+#define MODULE_VERSION    "2.1"
+#define MODULE_VERSION_SVN "$Id: smbnt.c 9239 2015-05-22 15:03:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -104,13 +104,14 @@
 #include <openssl/des.h>
 #include "hmacmd5.h"
 
+#define PORT_NBNS 137
 #define PORT_SMB 139
 #define PORT_SMBNT 445
 #define WIN2000_NATIVEMODE 1
 #define WIN_NETBIOSMODE 2
 #define PASSWORD 3
 #define HASH 4
-#define MACHINE 5
+#define MACHINE_NAME 5
 #define LOCAL 6
 #define NTDOMAIN 7
 #define BOTH 8
@@ -144,9 +145,9 @@
 #endif
 
 typedef struct __SMBNT_DATA {
-  char challenge[8];
-  unsigned char workgroup[16];
-  unsigned char workgroup_other[16];
+  unsigned char challenge[8];
+  char workgroup[16];
+  char workgroup_other[16];
   unsigned char machine_name[16];
   int security_mode;
   int authLevel;
@@ -168,6 +169,13 @@ enum MODULE_STATE
 int tryLogin(int hSocket, sLogin** login, _SMBNT_DATA* _psSessionData, char* szLogin, char* szPassword);
 int initModule(sLogin* login, _SMBNT_DATA *_psSessionData);
 char* parseFullyQualifiedUsername(_SMBNT_DATA *_psSessionData, char* szLogin);
+int NBSSessionRequest(int hSocket, _SMBNT_DATA* _psSessionData);
+int NBSTATQuery(sLogin *_psLogin,_SMBNT_DATA* _psSessionData);
+int SMBNegProt(int hSocket, _SMBNT_DATA* _psSessionData);
+
+extern void hmac_md5_init_limK_to_64(const unsigned char* key, int key_len, HMACMD5Context *ctx);
+extern void hmac_md5_update(const unsigned char *text, int text_len, HMACMD5Context *ctx);
+extern void hmac_md5_final(unsigned char *digest, HMACMD5Context *ctx);
 
 // Tell medusa how many parameters this module allows
 int getParamNumber()
@@ -242,6 +250,18 @@ void showUsage()
   writeVerbose(VB_NONE, "5: Testing an individual NTLM hash..."); 
   writeVerbose(VB_NONE, "    medusa -M smbnt -H hosts.txt -u administrator -p 92D887C8010492C2944E2DF489A880E4:7A2EDE4F51BC5A03984C6BA2C239CF63::: -m PASS:HASH");
   writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "Access level:");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "This module performs both an SMB authentication request (Session Setup AndX) and a ");
+  writeVerbose(VB_NONE, "share connection request (Tree Connect AndX). The share connection request is for the ");
+  writeVerbose(VB_NONE, "default hidden administrative share ADMIN$. The goal is to identify if the credentials ");
+  writeVerbose(VB_NONE, "being tested have administrative rights to the target system. The following examples ");
+  writeVerbose(VB_NONE, "highlight how to interrupt the responses.");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "  Valid administrative-level credentials: [SUCCESS (ADMIN$ - Access Allowed)]");
+  writeVerbose(VB_NONE, "  Valid user-level credentials: [SUCCESS (ADMIN$ - Access Denied)]");
+  writeVerbose(VB_NONE, "  Valid credentials, access level unknown: [SUCCESS (ADMIN$ - Share Unavailable)]");
+  writeVerbose(VB_NONE, "");
 }
 
 // The "main" of the medusa module world - this is what gets called to actually do the work
@@ -253,11 +273,11 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_SMBNT_DATA));  
   memset(psSessionData, 0, sizeof(_SMBNT_DATA));
 
-  if ( !(0 <= argc <= 3) )
+  if ((argc < 0) || (argc > 5))
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
-  } 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
+  }
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
@@ -315,7 +335,7 @@ int go(sLogin* logins, int argc, char *argv[])
         else if (strcmp(pOpt, "HASH") == 0)
           psSessionData->hashFlag = HASH;
         else if (strcmp(pOpt, "MACHINE") == 0)
-          psSessionData->hashFlag = MACHINE;
+          psSessionData->hashFlag = MACHINE_NAME;
         else
           writeError(ERR_WARNING, "Invalid value for method PASS.");
       }
@@ -332,7 +352,11 @@ int go(sLogin* logins, int argc, char *argv[])
         else if (strcmp(pOpt, "LMv2") == 0)
           psSessionData->authLevel = AUTH_LMv2;
         else if (strcmp(pOpt, "NTLMv2") == 0)
+        {
           psSessionData->authLevel = AUTH_NTLMv2;
+          /* NTLMv2 authentication is returning a STATUS_INVALID_PARAMETER response with 2012R2 servers. This issue pre-dates the AndX modification. */
+          writeError(ERR_FATAL, "NTLMv2 support currently disabled. The default authentication mode of LMv2 should work in all cases that NTLMv2 is required.");
+        }
         else
           writeError(ERR_WARNING, "Invalid value for method AUTH.");
       }
@@ -345,7 +369,7 @@ int go(sLogin* logins, int argc, char *argv[])
         writeError(ERR_WARNING, "Invalid method: %s.", pOpt);
       }
     
-      free(pOptTmp);
+      FREE(pOptTmp);
     }
  
     initModule(logins, psSessionData);
@@ -359,8 +383,6 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char *bufReceive = NULL;
-  char *strtok_ptr = NULL, *pOpt = NULL, *pOptTmp = NULL;
   char *szUser = NULL;
   sConnectParams params;
   sCredentialSet *psCredSet = NULL;
@@ -424,6 +446,10 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
 
         writeError(ERR_DEBUG_MODULE, "Connected");
  
+        if (NBSTATQuery(psLogin, _psSessionData) < 0) {
+          writeError(ERR_ERROR, "NetBIOS Name Query Failed with host: %s (proceeding anyways).", psLogin->psServer->pHostIP);
+        }
+        
         if (NBSSessionRequest(hSocket, _psSessionData) < 0) {
           writeError(ERR_ERROR, "Session Setup Failed with host: %s. Is the server service running?", psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
@@ -652,7 +678,7 @@ int HashLM(_SMBNT_DATA *_psSessionData, unsigned char **lmhash, unsigned char *p
     }
   } else {
     /* Password == Machine Name */
-    if (_psSessionData->hashFlag == MACHINE) {
+    if (_psSessionData->hashFlag == MACHINE_NAME) {
       for (i = 0; i < 16; i++) {
         if (_psSessionData->machine_name[i] > 0x39)
           _psSessionData->machine_name[i] = _psSessionData->machine_name[i] | 0x20;     /* convert upper case to lower */
@@ -661,7 +687,7 @@ int HashLM(_SMBNT_DATA *_psSessionData, unsigned char **lmhash, unsigned char *p
     }
     
     /* convert lower case characters to upper case */
-    strncpy(password, pass, 14);
+    strncpy((char *)password, (char *)pass, 14);
     for (i = 0; i < 14; i++) {
       if ((password[i] >= 0x61) && (password[i] <= 0x7a))      /* a - z */
         password[i] -= 0x20;
@@ -696,13 +722,12 @@ int MakeNTLM(_SMBNT_DATA *_psSessionData, unsigned char *ntlmhash, unsigned char
   MD4_CTX md4Context;
   unsigned char hash[16];                       /* MD4_SIGNATURE_SIZE = 16 */
   unsigned char unicodePassword[256 * 2];       /* MAX_NT_PASSWORD = 256 */
-  unsigned char p21[21];
-  unsigned char ntlm_response[24];
-  int i = 0, j = 0;
+  unsigned int i = 0, j = 0;
   int mdlen;
   unsigned char *p = NULL;
   char HexChar;
   int HexValue;
+  unsigned char NO_PASSWORD[1] = "";
 
   /* Use NTLM Hash instead of password */
   if (_psSessionData->hashFlag == HASH) {
@@ -724,8 +749,8 @@ int MakeNTLM(_SMBNT_DATA *_psSessionData, unsigned char *ntlmhash, unsigned char
     }
     else if (*p == 'N') {
       writeError(ERR_DEBUG_MODULE, "Found \"NO PASSWORD\" for NTLM Hash.");
-      pass = "";
-      
+      pass = NO_PASSWORD;
+ 
       /* Initialize the Unicode version of the secret (== password). */
       /* This implicitly supports 8-bit ISO8859/1 characters. */
       bzero(unicodePassword, sizeof(unicodePassword));
@@ -765,7 +790,7 @@ int MakeNTLM(_SMBNT_DATA *_psSessionData, unsigned char *ntlmhash, unsigned char
     }
   } else {
     /* Password == Machine Name */
-    if (_psSessionData->hashFlag == MACHINE) {
+    if (_psSessionData->hashFlag == MACHINE_NAME) {
       for (i = 0; i < 16; i++) {
         if (_psSessionData->machine_name[i] > 0x39)
           _psSessionData->machine_name[i] = _psSessionData->machine_name[i] | 0x20;     /* convert upper case to lower */
@@ -843,7 +868,8 @@ int HashLMv2(_SMBNT_DATA *_psSessionData, unsigned char **LMv2hash, unsigned cha
   unsigned char unicodeTarget[256 * 2];
   HMACMD5Context ctx;
   unsigned char kr_buf[16];
-  int ret, i;
+  int ret;
+  unsigned int i;
   unsigned char client_challenge[8] = { 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88 };
 
   memset(ntlm_hash, 0, 16);
@@ -897,7 +923,6 @@ int HashLMv2(_SMBNT_DATA *_psSessionData, unsigned char **LMv2hash, unsigned cha
     message authentication code algorithm is applied to this value using the 16-byte NTLMv2 hash 
     (calculated above) as the key. This results in a 16-byte output value.
   */
-
   hmac_md5_init_limK_to_64(kr_buf, 16, &ctx);
   hmac_md5_update(_psSessionData->challenge, 8, &ctx);
   hmac_md5_update(client_challenge, 8, &ctx);
@@ -942,7 +967,8 @@ int HashNTLMv2(_SMBNT_DATA *_psSessionData, unsigned char **NTLMv2hash, int *iBy
   unsigned char unicodeTarget[256 * 2];
   HMACMD5Context ctx;
   unsigned char kr_buf[16];
-  int ret, i, iTargetLen;
+  unsigned int i;
+  int ret, iTargetLen;
   unsigned char client_challenge[8] = { 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88 };
 
   /*
@@ -1091,6 +1117,7 @@ int HashNTLMv2(_SMBNT_DATA *_psSessionData, unsigned char **NTLMv2hash, int *iBy
   return SUCCESS;
 }
 
+
 /*
    NBS Session Request
    Function: Request a new session from the server
@@ -1101,23 +1128,37 @@ int NBSSessionRequest(int hSocket, _SMBNT_DATA* _psSessionData)
   char nb_name[32];             /* netbiosname */
   char nb_local[32];            /* netbios localredirector */
   unsigned char rqbuf[7] = { 0x81, 0x00, 0x00, 0x48, 0x20, 0x00, 0x20 };
-  char *buf = NULL;
-  char *bufReceive = NULL;
+  unsigned char *buf = NULL;
+  unsigned char *bufReceive = NULL;
   int nReceiveBufferSize = 0;
-  
+  int i = 0;  
+
   /* if we are running in native mode (aka port 445) don't do netbios */
   if (_psSessionData->protoFlag == WIN2000_NATIVEMODE)
     return 0;
-  
-  /* convert computer name to netbios name */
+
+  /* convert computer name to netbios name -- https://support.microsoft.com/kb/194203 */
   memset(nb_name, 0, 32);
   memset(nb_local, 0, 32);
-  memcpy(nb_name, "CKFDENECFDEFFCFGEFFCCACACACACACA", 32);      /* *SMBSERVER */
-  /* smbat/netbios.c/getNetbiosName */
-  /* memcpy(nb_local, "EHGFHCGHFDHFHIEMGPGCHDHEGFHCCACA", 32); */
+
+  if (_psSessionData->machine_name[0] == 0x00) {
+    writeVerbose(VB_GENERAL, "%s: NetBIOS calling name: *SMBSERVER", MODULE_NAME);
+    memcpy(nb_name, "CKFDENECFDEFFCFGEFFCCACACACACACA", 32);      /* *SMBSERVER */
+  }
+  else
+  {
+    writeVerbose(VB_GENERAL, "%s: NetBIOS calling name: %s", MODULE_NAME, _psSessionData->machine_name);
+    for (i = 0; i< 16; i++)
+    {
+      memset(nb_name + i * 2, ((_psSessionData->machine_name[i]) >> 4) + 0x41, 1);
+      memset(nb_name + i * 2 + 1, ((_psSessionData->machine_name[i]) & 0x0F) + 0x41, 1);
+    }
+  }
+  writeVerbose(VB_GENERAL, "%s: NetBIOS calling name: %s (encoded)", MODULE_NAME, nb_name);
+
   memcpy(nb_local, "ENEFEEFFFDEBCACACACACACACACACACA", 32);     /* MEDUSA */
   
-  buf = (char *) malloc(100);
+  buf = malloc(100);
   memset(buf, 0, 100);
   memcpy(buf, (char *) rqbuf, 5);
   memcpy(buf + 5, nb_name, 32);
@@ -1125,12 +1166,12 @@ int NBSSessionRequest(int hSocket, _SMBNT_DATA* _psSessionData)
   memcpy(buf + 39, nb_local, 32);
   memcpy(buf + 71, (char *) rqbuf + 5, 1);
 
-  if (medusaSend(hSocket, (char *) buf, 76, 0) < 0)
+  if (medusaSend(hSocket, buf, 76, 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
   }
 
-  free(buf);
+  FREE(buf);
 
   nReceiveBufferSize = 0;
   bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
@@ -1143,6 +1184,111 @@ int NBSSessionRequest(int hSocket, _SMBNT_DATA* _psSessionData)
     return FAILURE;                  /* failed */
 }
 
+/* NetBIOS Name Service Query */
+int NBSTATQuery(sLogin *_psLogin,_SMBNT_DATA* _psSessionData)
+{
+  sConnectParams params;
+  int hSocket;
+  int i = 0, j = 0;
+  int iResponseOffset = 56;
+  int iNameCount = 0;
+  int iNameType = 0;
+  char nb_name[16];
+
+  unsigned char nbstat[50] = {
+    0x81, 0xec, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x20, 0x43, 0x4b, 0x41,
+    0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
+    0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
+    0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
+    0x41, 0x41, 0x41, 0x41, 0x41, 0x00, 0x00, 0x21,
+    0x00, 0x01
+  };
+  
+  unsigned char *bufReceive = NULL;
+  int nReceiveBufferSize = 0;
+  
+  /* if we are running in native mode (aka port 445) don't do netbios */
+  if (_psSessionData->protoFlag == WIN2000_NATIVEMODE)
+    return 0;
+
+  memset(&params, 0, sizeof(sConnectParams));
+  initConnectionParams(_psLogin, &params);
+  params.nPort = PORT_NBNS;
+
+  hSocket = medusaConnectUDP(&params);
+  if (hSocket < 0)
+  {
+    writeError(ERR_ERROR, "[%s] Failed to connect to NBNS UDP port (%d). Auto-identification of NetBIOS name unsuccessful on host: %s.", MODULE_NAME, PORT_NBNS, _psLogin->psServer->pHostIP);
+    _psLogin->iResult = LOGIN_RESULT_UNKNOWN;
+    return FAILURE;
+  }
+
+  if (medusaSend(hSocket, nbstat, 50, 0) < 0)
+  {
+    writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
+    return FAILURE;
+  }
+
+  nReceiveBufferSize = 0;
+  bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
+  if ((bufReceive == NULL) || (nReceiveBufferSize < iResponseOffset + 1))
+    return FAILURE;
+ 
+  /* Find the primary domain/workgroup name */
+  /* name (15 bytes) + type (1 byte) + flages (2 bytes) */
+  iNameCount = bufReceive[iResponseOffset];
+  writeVerbose(VB_GENERAL, "%s: NBSTAT - Number of names: %d", MODULE_NAME, iNameCount);
+  if ((iResponseOffset + iNameCount * 16) > nReceiveBufferSize)
+  {
+    writeError(ERR_ERROR, "%s failed: unexpected response size to nbstat query.", MODULE_NAME);
+    return FAILURE;
+  }
+
+  iResponseOffset++;
+  for (i = 0; i < iNameCount; i++)
+  {
+    iNameType = bufReceive[iResponseOffset + 15];
+    memset(nb_name, 0, 16);
+    memcpy(nb_name, bufReceive + iResponseOffset, 16);
+
+    switch(iNameType)
+    {
+      case 0x00:
+        writeVerbose(VB_GENERAL, "%s: NetBIOS Name (workstation service name): %s", MODULE_NAME, nb_name);
+        break;
+      case 0x03:
+        writeVerbose(VB_GENERAL, "%s: NetBIOS Name (messenger service name): %s", MODULE_NAME, nb_name);
+        break;
+      case 0x1B:
+        writeVerbose(VB_GENERAL, "%s: NetBIOS Name (domain master browser name): %s", MODULE_NAME, nb_name);
+        break;
+      case 0x1C:
+        writeVerbose(VB_GENERAL, "%s: NetBIOS Name (domain group name): %s", MODULE_NAME, nb_name);
+        for (j = 0; j < 16; j++)
+          _psSessionData->workgroup[j] = nb_name[j];
+        break;
+      case 0x20:
+        writeVerbose(VB_GENERAL, "%s: NetBIOS Name (server service name): %s", MODULE_NAME, nb_name);
+        for (j = 0; j < 16; j++)
+          _psSessionData->machine_name[j] = nb_name[j];
+        break;
+      default:
+        writeVerbose(VB_GENERAL, "%s: NetBIOS Name (other type - %d): %s", MODULE_NAME,iNameType, nb_name);
+        break;
+    }
+
+    iResponseOffset += 18;
+  }
+
+  writeVerbose(VB_GENERAL, "%s: Server machine name: %s", MODULE_NAME, _psSessionData->machine_name);
+  writeVerbose(VB_GENERAL, "%s: Server primary domain: %s", MODULE_NAME, _psSessionData->workgroup);
+
+  FREE(bufReceive);
+
+  return SUCCESS;
+}
+
 
 /*
    SMBNegProt
@@ -1179,7 +1325,7 @@ int SMBNegProt(int hSocket, _SMBNT_DATA* _psSessionData)
     0x4c, 0x4d, 0x20, 0x30, 0x2e, 0x31, 0x32, 0x00
   };
 
-  char* bufReceive;
+  unsigned char* bufReceive;
   int nReceiveBufferSize = 0;
   int i = 0, j = 0;
   int iLength = 168;
@@ -1194,7 +1340,7 @@ int SMBNegProt(int hSocket, _SMBNT_DATA* _psSessionData)
     iResponseOffset = 65;
   }
 
-  if (medusaSend(hSocket, (char *) buf, iLength, 0) < 0)
+  if (medusaSend(hSocket, buf, iLength, 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -1226,7 +1372,7 @@ int SMBNegProt(int hSocket, _SMBNT_DATA* _psSessionData)
         writeError(ERR_ERROR, "%s: Server requested PLAINTEXT password. HASH password mode not supported for this configuration.", MODULE_NAME);
         return FAILURE;
       }
-      if (_psSessionData->hashFlag == MACHINE)
+      if (_psSessionData->hashFlag == MACHINE_NAME)
       {
         writeError(ERR_ERROR, "%s: Server requested PLAINTEXT password. MACHINE password mode not supported for this configuration.", MODULE_NAME);
         return FAILURE;
@@ -1252,9 +1398,6 @@ int SMBNegProt(int hSocket, _SMBNT_DATA* _psSessionData)
   memcpy(_psSessionData->challenge, (char *) bufReceive + iResponseOffset, sizeof(_psSessionData->challenge));
 
   /* Find the primary domain/workgroup name */
-  memset(_psSessionData->workgroup, 0, 16);
-  memset(_psSessionData->machine_name, 0, 16);
-
   while ((bufReceive[iResponseOffset + 8 + i * 2] != 0) && (i < 16)) {
     _psSessionData->workgroup[i] = bufReceive[iResponseOffset + 8 + i * 2];
     i++;
@@ -1268,6 +1411,8 @@ int SMBNegProt(int hSocket, _SMBNT_DATA* _psSessionData)
   writeVerbose(VB_GENERAL, "%s: Server machine name: %s", MODULE_NAME, _psSessionData->machine_name);
   writeVerbose(VB_GENERAL, "%s: Server primary domain: %s", MODULE_NAME, _psSessionData->workgroup);
 
+  FREE(bufReceive);
+
   return SUCCESS;
 }
 
@@ -1278,24 +1423,27 @@ int SMBNegProt(int hSocket, _SMBNT_DATA* _psSessionData)
             the server.
   Returns: TRUE on success else FALSE.
 */
-unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* szLogin, char* szPassword)
+unsigned long SMBSessionSetup(int hSocket, sLogin** psLogin, _SMBNT_DATA *_psSessionData, char* szLogin, char* szPassword)
 {
   unsigned char buf[512];
   unsigned char *LMv2hash = NULL;
   unsigned char *NTLMv2hash = NULL;
   unsigned char *NTLMhash = NULL;
   unsigned char *LMhash = NULL;
-  char *bufReceive = NULL;
+  unsigned char *bufReceive = NULL;
   int nReceiveBufferSize = 0;
-  int i, ret;
-  int iByteCount, iOffset;
+  int ret;
+  int iByteCount;
+  int iOffset = 0;
+  unsigned char szPath[256];
+  unsigned long SMBSessionRet;
   
   if (_psSessionData->accntFlag == LOCAL) {
     strcpy((char *) _psSessionData->workgroup, "localhost");
   } else if (_psSessionData->accntFlag == BOTH) {
     memset(_psSessionData->workgroup, 0, 16);
   } else if (_psSessionData->accntFlag == OTHER) {
-    strncpy((char *) _psSessionData->workgroup, _psSessionData->workgroup_other, 16);
+    strncpy(_psSessionData->workgroup, _psSessionData->workgroup_other, 16);
   }
 
   /* NetBIOS Session Service */
@@ -1314,10 +1462,10 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
     0x00, 0x00,                                       /* Process ID High */
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,   /* Signature */
     0x00, 0x00,                                       /* Reserved */
-    0x00, 0x00,                                       /* Tree ID */
+    0xFF, 0xFF,                                       /* Tree ID */
     0x13, 0x37,                                       /* Process ID */
     0x00, 0x00,                                       /* User ID */
-    0x01, 0x00                                        /* Multiplx ID */
+    0x01, 0x00                                        /* Multiplex ID */
   };
 
   memset(buf, 0, 512);
@@ -1333,7 +1481,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
 
       unsigned char szSessionRequest[23] = {
         0x0a,                             /* Word Count */
-        0xff,                             /* AndXCommand: No further commands */
+        0x75,                             /* AndXCommand: Tree Connect */
         0x00,                             /* Reserved */
         0x00, 0x00,                       /* AndXOffset */
         0xff, 0xff,                       /* Max Buffer */
@@ -1360,7 +1508,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
         return FAILURE;
 
       memcpy(buf + iOffset, LMhash, 24);
-      free(LMhash); 
+      FREE(LMhash); 
    
     }
     else if (_psSessionData->authLevel == AUTH_NTLM)
@@ -1369,7 +1517,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
     
       unsigned char szSessionRequest[29] = {
         0x0d,                             /* Word Count */
-        0xff,                             /* AndXCommand: No further commands */
+        0x75,                             /* AndXCommand: Tree Connect */
         0x00,                             /* Reserved */
         0x00, 0x00,                       /* AndXOffset */
         0xff, 0xff,                       /* Max Buffer */
@@ -1399,7 +1547,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
         return FAILURE;
 
       memcpy(buf + iOffset + 24, NTLMhash, 24); /* Skip space for LM hash */
-      free(NTLMhash);
+      FREE(NTLMhash);
     }
     else if (_psSessionData->authLevel == AUTH_LMv2)
     {
@@ -1407,7 +1555,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
     
       unsigned char szSessionRequest[29] = {
         0x0d,                             /* Word Count */
-        0xff,                             /* AndXCommand: No further commands */
+        0x75,                             /* AndXCommand: Tree Connect */
         0x00,                             /* Reserved */
         0x00, 0x00,                       /* AndXOffset */
         0xff, 0xff,                       /* Max Buffer */
@@ -1428,15 +1576,12 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
       memcpy(buf + 36, szSessionRequest, 29);
 
       /* Calculate and set LMv2 response hash */
-      LMv2hash = (unsigned char *) malloc(24);
-      memset(LMv2hash, 0, 24);
-
       ret = HashLMv2(_psSessionData, &LMv2hash, (unsigned char *) szLogin, (unsigned char *) szPassword);
       if (ret == FAILURE)
         return FAILURE;
 
       memcpy(buf + iOffset, LMv2hash, 24);
-      free(LMv2hash);
+      FREE(LMv2hash);
     }
     else if (_psSessionData->authLevel == AUTH_NTLMv2)
     {
@@ -1444,7 +1589,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
     
       unsigned char szSessionRequest[29] = {
         0x0d,                             /* Word Count */
-        0xff,                             /* AndXCommand: No further commands */
+        0x75,                             /* AndXCommand: Tree Connect */
         0x00,                             /* Reserved */
         0x00, 0x00,                       /* AndXOffset */
         0xff, 0xff,                       /* Max Buffer */
@@ -1469,7 +1614,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
         return FAILURE;
       
       memcpy(buf + iOffset, LMv2hash, 24);
-      free(LMv2hash);
+      FREE(LMv2hash);
 
       /* Calculate and set NTLMv2 response hash */
       ret = HashNTLMv2(_psSessionData, &NTLMv2hash, &iByteCount, (unsigned char *) szLogin, (unsigned char *) szPassword);
@@ -1481,7 +1626,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
       writeError(ERR_DEBUG_MODULE, "HashNTLMv2 response length: %d", iByteCount);
 
       memcpy(buf + iOffset + 24, NTLMv2hash, iByteCount);
-      free(NTLMv2hash);
+      FREE(NTLMv2hash);
 
       iByteCount += 24; /* Reflects length of both LMv2 and NTLMv2 responses */
     }
@@ -1492,7 +1637,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
 
     unsigned char szSessionRequest[23] = {
       0x0a,                             /* Word Count */
-      0xff,                             /* AndXCommand: No further commands */
+      0x75,                             /* AndXCommand: Tree Connect */
       0x00,                             /* Reserved */
       0x00, 0x00,                       /* AndXOffset */
       0xff, 0xff,                       /* Max Buffer */
@@ -1527,7 +1672,7 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
       is "password level = 0". This results in only two attempts to being made by the 
       remote server; the password as is and the password in all-lower case.
     */
-    strncpy(buf + iOffset, szPassword, 256);
+    strncpy((char *)buf + iOffset, szPassword, 256);
   }
   else
   {
@@ -1542,21 +1687,65 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
   iByteCount += strlen((char *) _psSessionData->workgroup) + 1; /* NULL pad workgroup name */
 
   /* Set native OS and LAN Manager values */
-  sprintf(buf + iOffset + iByteCount, "Unix"); 
+  sprintf((char *)buf + iOffset + iByteCount, "Unix"); 
   iByteCount += strlen("Unix") + 1; /* NULL pad OS name */
-  sprintf(buf + iOffset + iByteCount, "Samba"); 
+  sprintf((char *)buf + iOffset + iByteCount, "Samba"); 
   iByteCount += strlen("Samba") + 1; /* NULL pad LAN Manager name */
 
-  /* Set the header length */
-  buf[2] = (iOffset - 4 + iByteCount) / 256;
-  buf[3] = (iOffset - 4 + iByteCount) % 256;
-  writeError(ERR_DEBUG_MODULE, "[%s] Set NBSS header length: %2.2X", MODULE_NAME, buf[3]);
-
   /* Set data byte count */
   buf[iOffset - 2] = iByteCount;
   writeError(ERR_DEBUG_MODULE, "[%s] Set byte count: %2.2X", MODULE_NAME, buf[57]);
 
-  if (medusaSend(hSocket, (char *) buf, iOffset + iByteCount, 0) < 0)
+  /* Set AndXOffset */
+  buf[39] = (iOffset - 4 + iByteCount) % 256;
+  buf[40] = (iOffset - 4 + iByteCount) / 256;
+
+  /* Chained Tree AndX Request - Test for ADMIN$ access */
+  iOffset += iByteCount;
+
+  unsigned char szTreeConnectRequest[9] = {
+    0x04,                               /* Word Count */
+    0xff,                               /* AndXCommand: No further commands */
+    0x00,                               /* Reserved */
+    0x00, 0x00,                         /* AndXOffset */
+    0x08, 0x00,                         /* Flags */
+    0x01, 0x00                          /* Password Length */
+  };
+
+  memcpy(buf + iOffset, szTreeConnectRequest, 9);
+  iOffset += 9;
+
+  /* Set byte count (BCC) */
+  /* Password (1) + "\\" + "host IP" + "\ADMIN$" + null termination + service (6) */
+  iByteCount = 1 + 2 + strlen((char *) (*psLogin)->psServer->pHostIP) + 7 + 1 + 6;
+  buf[iOffset] = (iByteCount) % 256;
+  buf[iOffset + 1] = (iByteCount) / 256;
+  writeError(ERR_DEBUG_MODULE, "[%s] Set byte count (BCC): 0x%2.2X%2.2X", MODULE_NAME, buf[iOffset + 1], buf[iOffset]);
+  iOffset += 2;
+
+  /* Set password field */
+  memset(buf + iOffset, 0, 1);
+  iOffset++;
+ 
+  /* Set target path -- e.g., \\192.168.0.1\ADMIN$ */
+  memset(szPath, 0, 256);
+  snprintf((char *)szPath, sizeof(szPath), "\\\\%s\\ADMIN$", (*psLogin)->psServer->pHostIP);
+
+  memcpy(buf + iOffset, szPath, strlen((char *)szPath));
+  iOffset += strlen((char *)szPath) + 1;
+
+  /* Set service field */
+  unsigned char szService[6] = { 0x3F, 0x3F, 0x3F, 0x3F, 0x3F, 0x00 };
+  memcpy(buf + iOffset, szService, 6);
+  iOffset += 6;
+  /* End Chained Tree AndX Request */
+  
+  /* Set the header length */
+  buf[2] = (iOffset - 4) / 256;
+  buf[3] = (iOffset - 4) % 256;
+  writeError(ERR_DEBUG_MODULE, "[%s] Set NBSS header length: %2.2X", MODULE_NAME, buf[3]);
+
+  if (medusaSend(hSocket, buf, iOffset, 0) < 0)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -1568,7 +1757,10 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
  
   /* 41 - Action (Guest/Non-Guest Account) */
   /*  9 - NT Status (Error code) */
-  return (((bufReceive[41] & 0x01) << 24) | ((bufReceive[11] & 0xFF) << 16) | ((bufReceive[10] & 0xFF) << 8) | (bufReceive[9] & 0xFF));
+  SMBSessionRet = ((bufReceive[41] & 0x01) << 24) | ((bufReceive[11] & 0xFF) << 16) | ((bufReceive[10] & 0xFF) << 8) | (bufReceive[9] & 0xFF);
+  FREE(bufReceive);
+
+  return SMBSessionRet;
 }
 
 int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* szLogin, char* szPassword)
@@ -1577,7 +1769,8 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
   unsigned long SMBSessionRet;
   char *pErrorMsg = NULL;
   char ErrorCode[10];
-  int i, iRet;
+  int iRet;
+  unsigned int i;
 
   /* Nessus Plugins: smb_header.inc */
   /* Note: we are currently only examining the lower 2 bytes of data */
@@ -1597,12 +1790,15 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
     0xC000015B,         /* STATUS_LOGON_TYPE_NOT_GRANTED */
     0xC000018D,         /* STATUS_TRUSTED_RELATIONSHIP_FAILURE */
     0xC0000193,         /* STATUS_ACCOUNT_EXPIRED */
+    0xC0000199,         /* STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT */
     0x00BF0002,         /* STATUS_ACCOUNT_EXPIRED_OR_DISABLED (LM Authentication) */
     0xC0000224,         /* STATUS_PASSWORD_MUST_CHANGE */
     0x00C20002,         /* STATUS_PASSWORD_MUST_CHANGE (LM Authentication) */
     0xC0000234,         /* STATUS_ACCOUNT_LOCKED_OUT (No LM Authentication Code) */
     0x00050001,         /* AS400_STATUS_LOGON_FAILURE */
-    0x00000064          /* The machine you are logging onto is protected by an authentication firewall. */
+    0x00000064,         /* The machine you are logging onto is protected by an authentication firewall. */
+    0xC0000022,         /* STATUS_ACCESS_DENIED */
+    0xC00000CC          /* STATUS_BAD_NETWORK_NAME */
   };
 
   char *smbErrorMsg[] = {
@@ -1621,17 +1817,20 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
     "STATUS_LOGON_TYPE_NOT_GRANTED",
     "STATUS_TRUSTED_RELATIONSHIP_FAILURE",
     "STATUS_ACCOUNT_EXPIRED",
+    "STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT",
     "STATUS_ACCOUNT_EXPIRED_OR_DISABLED (LM)",
     "STATUS_PASSWORD_MUST_CHANGE",
     "STATUS_PASSWORD_MUST_CHANGE (LM)",
     "STATUS_ACCOUNT_LOCKED_OUT",
     "AS400_STATUS_LOGON_FAILURE",
-    "AUTHENTICATION_FIREWALL_PROTECTION"
+    "AUTHENTICATION_FIREWALL_PROTECTION",
+    "STATUS_ACCESS_DENIED",
+    "STATUS_BAD_NETWORK_NAME"
   };
 
   memset(&ErrorCode, 0, 10);
 
-  SMBSessionRet = SMBSessionSetup(hSocket, _psSessionData, szLogin, szPassword);
+  SMBSessionRet = SMBSessionSetup(hSocket, psLogin, _psSessionData, szLogin, szPassword);
   SMBerr = (unsigned long) SMBSessionRet & 0x00FFFFFF;
   SMBaction = ((unsigned long) SMBSessionRet & 0xFF000000) >> 24;
 
@@ -1662,8 +1861,13 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
         (*psLogin)->iResult = LOGIN_RESULT_ERROR;
       }
       else
+      {
+        (*psLogin)->pErrorMsg = malloc( 23 + 1 );
+        memset((*psLogin)->pErrorMsg, 0, 23 + 1 );
+        sprintf((*psLogin)->pErrorMsg, "ADMIN$ - Access Allowed");
         (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
-      
+      }      
+
       iRet = MSTATE_EXITING;
       break;
     case 0x00006F:  /* STATUS_INVALID_LOGON_HOURS - Valid password */
@@ -1673,13 +1877,16 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
                        authenticate to the machine." */
     case 0x000070:  /* STATUS_INVALID_WORKSTATION - Valid password */
     case 0xC00002:  /* STATUS_INVALID_WORKSTATION - Valid password (LM) */
-    case 0x000072:  /* STATUS_ACCOUNT_DISABLED - Valid password */
+    // TODO: Verify whether we can determine password validity from DISABLED. Win7 doesn't seem to tell us...
+    //case 0x000072:  /* STATUS_ACCOUNT_DISABLED - Valid password */
     case 0x000193:  /* STATUS_ACCOUNT_EXPIRED - Valid password */
     case 0xBF0002:  /* STATUS_ACCOUNT_DISABLED or STATUS_ACCOUNT_EXPIRED - Valid password (LM) */
     case 0x000224:  /* STATUS_PASSWORD_MUST_CHANGE - Valid password */
     case 0xC20002:  /* STATUS_PASSWORD_MUST_CHANGE - Valid password (LM) */
-    case 0x00006E:  /* Valid password, GPO Disabling Remote Connections Using NULL Passwords */
-    case 0x00015B:  /* Valid password, GPO "Deny access to this computer from the network" */
+    // TODO: Verify whether we can determine password validity from STATUS_ACCOUNT_RESTRICTION. Win7 doesn't seem to tell us...
+    //case 0x00006E:  /* Valid password, GPO Disabling Remote Connections Using NULL Passwords */
+    // TODO: Verify whether we can determine password validity from STATUS_ACCOUNT_RESTRICTION. Win7 doesn't seem to tell us...
+    //case 0x00015B:  /* Valid password, GPO "Deny access to this computer from the network" */
     case 0x000071:  /* Valid password, password expired and must be changed on next logon */
       (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
       sprintf(ErrorCode, "0x%6.6X:", SMBerr);
@@ -1689,6 +1896,20 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
       strncat((*psLogin)->pErrorMsg, pErrorMsg, strlen(pErrorMsg));
       iRet = MSTATE_EXITING;
       break;
+    case 0x000022:  /* Valid password, no access to ADMIN$ (non-administative account) */
+      (*psLogin)->pErrorMsg = malloc( 22 + 1 );
+      memset((*psLogin)->pErrorMsg, 0, 22 + 1 );
+      sprintf((*psLogin)->pErrorMsg, "ADMIN$ - Access Denied");
+      (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
+      iRet = MSTATE_EXITING;
+      break;
+    case 0x0000CC:  /* Valid password, but ADMIN$ not found (STATUS_BAD_NETWORK_NAME) */
+      (*psLogin)->pErrorMsg = malloc( 26 + 1 );
+      memset((*psLogin)->pErrorMsg, 0, 26 + 1 );
+      sprintf((*psLogin)->pErrorMsg, "ADMIN$ - Share Unavailable");
+      (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
+      iRet = MSTATE_EXITING;
+      break;
     case 0x050001:  /* AS/400 -- Incorrect password */
       writeError(ERR_DEBUG_MODULE, "[%s] AS/400 Access is Denied. Incorrect password or disabled account.", MODULE_NAME);
       (*psLogin)->iResult = LOGIN_RESULT_FAIL;
@@ -1709,14 +1930,14 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
       break;
   }
 
-  if (_psSessionData->hashFlag == MACHINE) { 
-    setPassResult((*psLogin), _psSessionData->machine_name);
+  if (_psSessionData->hashFlag == MACHINE_NAME) { 
+    setPassResult((*psLogin), (char *)_psSessionData->machine_name);
     iRet = MSTATE_EXITING;
   }
   else {
     setPassResult((*psLogin), szPassword);
   }
-
+ 
   return(iRet);
 }
 
