@@ -39,7 +39,7 @@
 #define MODULE_AUTHOR  "pmonkey@foofus.net"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SMTP Authentication with TLS"
 #define MODULE_VERSION    "2.0"
-#define MODULE_VERSION_SVN "$Id: smtp.c 1583 2011-07-11 18:51:19Z jmk $"
+#define MODULE_VERSION_SVN "$Id: smtp.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
@@ -132,11 +132,11 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_MODULE_DATA));
   memset(psSessionData, 0, sizeof(_MODULE_DATA));
 
-  if ( !(0 <= argc <= 3) )
+  if ((argc < 0) || (argc > 3))
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
-  } 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d).", MODULE_NAME, argc);
+    return FAILURE;
+  }
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
@@ -341,7 +341,6 @@ int initModule(sLogin* psLogin, _MODULE_DATA *_psSessionData)
 
 int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *params)
 { 
-  int iRet;
   unsigned char *bufSend = NULL;
   unsigned char *bufReceive = NULL;
   int nReceiveBufferSize = 0;
@@ -362,9 +361,9 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   nSendBufferSize = 5 + strlen(_psSessionData->szEHLO) + 2;
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
-  sprintf(bufSend, "EHLO %s\r\n", _psSessionData->szEHLO);
+  sprintf((char *)bufSend, "EHLO %s\r\n", _psSessionData->szEHLO);
   
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
     FREE(bufSend); 
@@ -373,7 +372,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   FREE(bufSend); 
  
   nReceiveBufferSize = 0;
-  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^250 .*\r\n|^250-.*\r\n") == FAILURE) || (bufReceive == NULL))
+  if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "250 .*\r\n") == FAILURE) || (bufReceive == NULL))
   {
     writeError(ERR_ERROR, "[%s] failed: Server did not respond with '250'. Exiting...", MODULE_NAME);
     FREE(bufReceive);
@@ -381,7 +380,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   }
 
   /* If server supports STARTTLS and we are not already within a SSL connection, let's use it. */
-  if ((params->nUseSSL == 0) && (strstr(bufReceive, "STARTTLS") != NULL))
+  if ((params->nUseSSL == 0) && (strstr((char *)bufReceive, "STARTTLS") != NULL))
   {
     FREE(bufReceive);
   
@@ -389,8 +388,8 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   
     bufSend = malloc(10 + 1);
     memset(bufSend, 0, 10 + 1);
-    sprintf(bufSend, "STARTTLS\r\n");
-    if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+    sprintf((char *)bufSend, "STARTTLS\r\n");
+    if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
     {
       writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
       FREE(bufSend);
@@ -421,9 +420,9 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
       nSendBufferSize = 5 + strlen(_psSessionData->szEHLO) + 2;
       bufSend = malloc(nSendBufferSize + 1);
       memset(bufSend, 0, nSendBufferSize + 1);
-      sprintf(bufSend, "EHLO %s\r\n", _psSessionData->szEHLO);
+      sprintf((char *)bufSend, "EHLO %s\r\n", _psSessionData->szEHLO);
   
-      if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+      if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
       {
         writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
         FREE(bufSend); 
@@ -432,7 +431,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
       FREE(bufSend); 
  
       nReceiveBufferSize = 0;
-      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "^250 .*\r\n|^250-.*\r\n") == FAILURE) || (bufReceive == NULL))
+      if ((medusaReceiveRegex(hSocket, &bufReceive, &nReceiveBufferSize, "250 .*\r\n") == FAILURE) || (bufReceive == NULL))
       {
         writeError(ERR_ERROR, "[%s] failed: Server did not respond with '250'. Exiting...", MODULE_NAME);
         FREE(bufReceive);
@@ -446,29 +445,29 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Ignoring server requested AUTH type and using user-specified value.", MODULE_NAME);
   }
-  else if ((strstr(bufReceive,"AUTH=LOGIN") != NULL))
+  else if ((strstr((char *)bufReceive, "AUTH=LOGIN") != NULL))
   {
     writeError(ERR_DEBUG_MODULE, "Detected authentication type: LOGIN");
     _psSessionData->nAuthType = AUTH_LOGIN;
   }
-  else if ((strstr(bufReceive,"AUTH=PLAIN") != NULL))
+  else if ((strstr((char *)bufReceive, "AUTH=PLAIN") != NULL))
   {
     writeError(ERR_DEBUG_MODULE, "Detected authentication type: PLAIN");
     _psSessionData->nAuthType = AUTH_PLAIN;
   }
-  else if ((strstr(bufReceive,"AUTH ") != NULL))
+  else if ((strstr((char *)bufReceive, "AUTH ") != NULL))
   {
-    if ((strstr(bufReceive,"LOGIN") != NULL))
+    if ((strstr((char *)bufReceive, "LOGIN") != NULL))
     {
       writeError(ERR_DEBUG_MODULE, "Detected authentication type: LOGIN");
       _psSessionData->nAuthType = AUTH_LOGIN;
     }
-    else if ((strstr(bufReceive,"PLAIN") != NULL))
+    else if ((strstr((char *)bufReceive, "PLAIN") != NULL))
     {
       writeError(ERR_DEBUG_MODULE, "Detected authentication type: PLAIN");
       _psSessionData->nAuthType = AUTH_PLAIN;
     }
-    else if ((strstr(bufReceive,"NTLM") != NULL))
+    else if ((strstr((char *)bufReceive, "NTLM") != NULL))
     {
       writeError(ERR_DEBUG_MODULE, "Detected authentication type: NTLM");
       _psSessionData->nAuthType = AUTH_NTLM;
@@ -492,7 +491,7 @@ int initConnection(_MODULE_DATA *_psSessionData, int hSocket, sConnectParams *pa
   C: AHdlbGRvbgB3M2xkMG4=
   S: 235 2.0.0 OK Authenticated
 */
-int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char* szPassword)
+int sendAuthPLAIN(int hSocket, char* szLogin, char* szPassword)
 {
   unsigned char* bufReceive = NULL;
   unsigned char* bufSend = NULL;
@@ -500,16 +499,15 @@ int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   unsigned char* szTmpBuf64 = NULL;
   int nSendBufferSize = 0;
   int nReceiveBufferSize = 0;
-  int nRet = SUCCESS;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Initiating PLAIN Authentication Attempt.", MODULE_NAME);
 
   /* --- Send initial AUTH PLAIN command --- */
   bufSend = malloc(12 + 1);
   memset(bufSend, 0, 12 + 1);
-  sprintf(bufSend, "AUTH PLAIN\r\n");
+  sprintf((char *)bufSend, "AUTH PLAIN\r\n");
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -528,22 +526,22 @@ int sendAuthPLAIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   nSendBufferSize = strlen(szLogin) + 1 + strlen(szLogin) + 1 + strlen(szPassword);
   szTmpBuf = malloc(nSendBufferSize + 1);
   memset(szTmpBuf, 0, nSendBufferSize + 1);
-  strncpy(szTmpBuf, szLogin, strlen(szLogin));
-  strncpy(szTmpBuf + strlen(szLogin) + 1, szLogin, strlen(szLogin));
-  strncpy(szTmpBuf + strlen(szLogin) + 1 + strlen(szLogin) + 1, szPassword, strlen(szPassword));
+  strncpy((char *)szTmpBuf, szLogin, strlen((char *)szLogin));
+  strncpy((char *)szTmpBuf + strlen(szLogin) + 1, szLogin, strlen(szLogin));
+  strncpy((char *)szTmpBuf + strlen(szLogin) + 1 + strlen(szLogin) + 1, szPassword, strlen(szPassword));
 
   szTmpBuf64 = malloc((2 * nSendBufferSize + 2) + 1);
   memset(szTmpBuf64, 0, (2 * nSendBufferSize + 2) + 1);
-  base64_encode(szTmpBuf, nSendBufferSize, szTmpBuf64);
+  base64_encode((char *)szTmpBuf, nSendBufferSize, (char *)szTmpBuf64);
   FREE(szTmpBuf);
 
-  bufSend = malloc(strlen(szTmpBuf64) + 2 + 1);
-  memset(bufSend, 0, strlen(szTmpBuf64) + 2 + 1);
+  bufSend = malloc(strlen((char *)szTmpBuf64) + 2 + 1);
+  memset(bufSend, 0, strlen((char *)szTmpBuf64) + 2 + 1);
 
-  sprintf(bufSend, "%s\r\n", szTmpBuf64);
+  sprintf((char *)bufSend, "%s\r\n", szTmpBuf64);
   FREE(szTmpBuf64);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -570,18 +568,16 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   unsigned char* szTmpBuf = NULL;
   unsigned char* szTmpBuf2 = NULL;
   unsigned char* szLoginDomain = NULL;
-  int nSendBufferSize = 0;
   int nReceiveBufferSize = 0;
-  int nRet = SUCCESS;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Initiating LOGIN Authentication Attempt.", MODULE_NAME);
 
   /* --- Send initial AUTH LOGIN command --- */
   bufSend = malloc(12 + 1);
   memset(bufSend, 0, 12 + 1);
-  sprintf(bufSend, "AUTH LOGIN\r\n");
+  sprintf((char *)bufSend, "AUTH LOGIN\r\n");
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -596,7 +592,7 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
     return FAILURE;
   }
 
-  if (((szTmpBuf = (char*)strstr(bufReceive, "334 ")) == NULL) || ((szTmpBuf2 = (char*)index(szTmpBuf, '\r')) == NULL))
+  if (((szTmpBuf = (unsigned char *)strstr((char *)bufReceive, "334 ")) == NULL) || ((szTmpBuf2 = (unsigned char *)index((char *)szTmpBuf, '\r')) == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server sent unexpected response to AUTH LOGIN request.", MODULE_NAME);
     return FAILURE;
@@ -605,10 +601,10 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   szTmpBuf2[0] = '\0';
 
   szTmpBuf += 4;
-  szPrompt = malloc(strlen(szTmpBuf) + 1);
-  memset(szPrompt, 0, strlen(szTmpBuf) + 1);
+  szPrompt = malloc(strlen((char *)szTmpBuf) + 1);
+  memset(szPrompt, 0, strlen((char *)szTmpBuf) + 1);
 
-  base64_decode(szTmpBuf, szPrompt);
+  base64_decode((char *)szTmpBuf, (char *)szPrompt);
   FREE(bufReceive);
 
   writeError(ERR_DEBUG_MODULE, "[%s] SMTP server sent the following prompt: %s", MODULE_NAME, szPrompt);
@@ -622,22 +618,22 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
     /* DOMAIN\USERNAME */
     szLoginDomain = malloc(strlen(_psSessionData->szDomain) + 1 + strlen(szLogin) + 1);
     memset(szLoginDomain, 0, strlen(_psSessionData->szDomain) + 1 + strlen(szLogin) + 1);
-    sprintf(szLoginDomain, "%s\\%s", _psSessionData->szDomain, szLogin); 
+    sprintf((char *)szLoginDomain, "%s\\%s", _psSessionData->szDomain, szLogin); 
   }
   else
-    szLoginDomain = szLogin;
+    szLoginDomain = (unsigned char *)szLogin;
 
   writeError(ERR_DEBUG_MODULE, "[%s] Sending authenticate login value: %s %s", MODULE_NAME, szLoginDomain, szPassword);
   
-  bufSend = malloc((2 * strlen(szLoginDomain) + 2) + 2 + 1);
-  memset(bufSend, 0, (2 * strlen(szLoginDomain) + 2) + 2 + 1);
-  base64_encode(szLoginDomain, strlen(szLoginDomain), bufSend);
-  strncat(bufSend, "\r\n", 2);
+  bufSend = malloc((2 * strlen((char *)szLoginDomain) + 2) + 2 + 1);
+  memset(bufSend, 0, (2 * strlen((char *)szLoginDomain) + 2) + 2 + 1);
+  base64_encode((char *)szLoginDomain, strlen((char *)szLoginDomain), (char *)bufSend);
+  strncat((char *)bufSend, "\r\n", 2);
   
   if (_psSessionData->szDomain)
     FREE(szLoginDomain);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -651,7 +647,7 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
     return FAILURE;
   }
 
-  if (((szTmpBuf = (char*)strstr(bufReceive, "334 ")) == NULL) || ((szTmpBuf2 = (char*)index(szTmpBuf, '\r')) == NULL))
+  if (((szTmpBuf = (unsigned char *)strstr((char *)bufReceive, "334 ")) == NULL) || ((szTmpBuf2 = (unsigned char *)index((char *)szTmpBuf, '\r')) == NULL))
   {
     writeError(ERR_ERROR, "[%s] SMTP server sent unexpected response to AUTH LOGIN request.", MODULE_NAME);
     return FAILURE;
@@ -660,10 +656,10 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   szTmpBuf2[0] = '\0';
 
   szTmpBuf += 4;
-  szPrompt = malloc(strlen(szTmpBuf) + 1);
-  memset(szPrompt, 0, strlen(szTmpBuf) + 1);
+  szPrompt = malloc(strlen((char *)szTmpBuf) + 1);
+  memset(szPrompt, 0, strlen((char *)szTmpBuf) + 1);
 
-  base64_decode(szTmpBuf, szPrompt);
+  base64_decode((char *)szTmpBuf, (char *)szPrompt);
   FREE(bufReceive);
 
   writeError(ERR_DEBUG_MODULE, "[%s] SMTP server sent the following prompt: %s", MODULE_NAME, szPrompt);
@@ -674,10 +670,10 @@ int sendAuthLOGIN(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   /* Base64 encoded value can be up to 2x+2 original text. Leave additional space for "\r\n" and NULL */
   bufSend = malloc((2 * strlen(szPassword) + 2) + 2 + 1);
   memset(bufSend, 0, (2 * strlen(szPassword) + 2) + 2 + 1);
-  base64_encode(szPassword, strlen(szPassword), bufSend);
-  strncat(bufSend, "\r\n", 2);
+  base64_encode((char *)szPassword, strlen((char *)szPassword), (char *)bufSend);
+  strncat((char *)bufSend, "\r\n", 2);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] Failed: medusaSend was not successful", MODULE_NAME);
   }
@@ -709,17 +705,17 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   szTmpBuf64 = malloc(2 * SmbLength(&sTmpReq) + 2);
   memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpReq) + 2);
 
-  base64_encode((char *)&sTmpReq, SmbLength(&sTmpReq), szTmpBuf64);
+  base64_encode((char *)&sTmpReq, SmbLength(&sTmpReq), (char *)szTmpBuf64);
   writeError(ERR_DEBUG_MODULE, "[%s] Sending initial challenge (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
-  nSendBufferSize = strlen(szTmpBuf64) + 2;
+  nSendBufferSize = strlen((char *)szTmpBuf64) + 2;
   bufSend = malloc(10 + nSendBufferSize + 1);
   memset(bufSend, 0, 10 + nSendBufferSize + 1);
-  sprintf(bufSend, "AUTH NTLM %s\r\n", szTmpBuf64);
+  sprintf((char *)bufSend, "AUTH NTLM %s\r\n", szTmpBuf64);
 
   FREE(szTmpBuf64);
 
-  if (medusaSend(hSocket, bufSend, strlen(bufSend), 0) < 0)
+  if (medusaSend(hSocket, bufSend, strlen((char *)bufSend), 0) < 0)
   {
     writeError(ERR_ERROR, "[%s] failed: medusaSend was not successful", MODULE_NAME);
     return FAILURE;
@@ -736,11 +732,11 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
     return FAILURE;
   }
 
-  szTmpBuf = ((char*)index(bufReceive, '\r'));
+  szTmpBuf = (unsigned char *)index((char *)bufReceive, '\r');
   szTmpBuf[0] = '\0';
 
   writeError(ERR_DEBUG_MODULE, "[%s] NTLM Challenge (B64 Encoded): %s", MODULE_NAME, bufReceive + 4);
-  base64_decode(bufReceive + 4, (char *)&sTmpChall);
+  base64_decode((char *)bufReceive + 4, (char *)&sTmpChall);
 
   FREE(bufReceive);
 
@@ -750,13 +746,13 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   szTmpBuf64 = malloc(2 * SmbLength(&sTmpResp) + 2);
   memset(szTmpBuf64, 0, 2 * SmbLength(&sTmpResp) + 2);
 
-  base64_encode((char *)&sTmpResp, SmbLength(&sTmpResp), szTmpBuf64);
+  base64_encode((char *)&sTmpResp, SmbLength(&sTmpResp), (char *)szTmpBuf64);
   writeError(ERR_DEBUG_MODULE, "[%s] NTLM Response (B64 Encoded): %s", MODULE_NAME, szTmpBuf64);
 
-  nSendBufferSize = strlen(szTmpBuf64) + 2;
+  nSendBufferSize = strlen((char *)szTmpBuf64) + 2;
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
-  sprintf(bufSend, "%s\r\n", szTmpBuf64);
+  sprintf((char *)bufSend, "%s\r\n", szTmpBuf64);
 
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
   {
@@ -780,7 +776,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
   {
     case AUTH_PLAIN:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending PLAIN Authentication.", MODULE_NAME);
-      nRet = sendAuthPLAIN(hSocket, _psSessionData, szLogin, szPassword);
+      nRet = sendAuthPLAIN(hSocket, szLogin, szPassword);
       break;
     case AUTH_LOGIN:
       writeError(ERR_DEBUG_MODULE, "[%s] Sending LOGIN Authentication.", MODULE_NAME);
@@ -812,27 +808,27 @@ int tryLogin(int hSocket, sLogin** psLogin, _MODULE_DATA* _psSessionData, char*
     nRet = MSTATE_EXITING;
   }
 
-  if (strstr(bufReceive,"235 ") != NULL)
+  if (strstr((char *)bufReceive, "235 ") != NULL)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt successful.", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_SUCCESS;
     nRet = MSTATE_EXITING;
   }
   /* 435 Unable to authenticate at present: Authentication Failure */
-  else if (strstr(bufReceive,"435 ") != NULL)
+  else if (strstr((char *)bufReceive, "435 ") != NULL)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed (435).", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
     nRet = MSTATE_RUNNING;
   }
   /* GroupWise - 501 Authentication failed! */
-  else if (strstr(bufReceive,"501 ") != NULL)
+  else if (strstr((char *)bufReceive, "501 ") != NULL)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed (501).", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
     nRet = MSTATE_RUNNING;
   }
-  else if (strstr(bufReceive,"535 ") != NULL)
+  else if (strstr((char *)bufReceive, "535 ") != NULL)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Login attempt failed (535).", MODULE_NAME);
     (*psLogin)->iResult = LOGIN_RESULT_FAIL;
