@@ -38,7 +38,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SNMP Community Strings"
 #define MODULE_VERSION    "2.1"
-#define MODULE_VERSION_SVN "$Id: snmp.c 1399 2010-08-04 21:04:25Z jmk $"
+#define MODULE_VERSION_SVN "$Id: snmp.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 
@@ -67,7 +67,8 @@ enum MODULE_STATE
   MSTATE_NEW,
   MSTATE_RUNNING,
   MSTATE_EXITING,
-  MSTATE_COMPLETE
+  MSTATE_COMPLETE,
+  MSTATE_FAILURE
 };
 
 // Forward declarations
@@ -151,11 +152,11 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData->nReadTimeout = RECEIVE_DELAY;
   psSessionData->nSendDelay = SEND_DELAY;
 
-  if ( !(0 <= argc <= 3) )
+  if ((argc < 0) || (argc > 4))
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
-  } 
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
+  }
   else 
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
@@ -286,7 +287,7 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
         break;
       case MSTATE_RUNNING:
         nState = sendRead(hSocket, _psSessionData, psCredSet->pPass);
-        if (nState == FAILURE)
+        if (nState == MSTATE_FAILURE)
         {
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
           return FAILURE;
@@ -347,6 +348,7 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
   }
 
   /* check if server responded to GET queries */
+  writeError(ERR_DEBUG_MODULE, "[%s] Checking for server responses.", MODULE_NAME);
   if (receiveRequest(hSocket, _psSessionData, &nPassCount, &arrszPassList, &szLocation) == FAILURE)
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Failed to find valid READ community string.", MODULE_NAME);
@@ -424,7 +426,7 @@ int initModule(sLogin* psLogin, _SNMP_DATA *_psSessionData)
   Sequence: 0x30 0x82 0x00 0x2f
   Version:  0x02 0x01 0x00
 */
-int parseLength(int nReceiveBufferSize, unsigned char* bufReceive)
+int parseLength(unsigned char* bufReceive)
 {
   int nLength = 0;
   int nOctets = 0;
@@ -478,7 +480,7 @@ int countResponses(int nReceiveBufferSize, unsigned char* bufReceive)
   for (i = 0; i < nReceiveBufferSize; i++) {
     if (bufReceive[i] == 0x30)
     {
-      nLength = parseLength(nReceiveBufferSize - i, &bufReceive[i]);
+      nLength = parseLength(&bufReceive[i]);
       if (nLength > 0)
       {
         writeError(ERR_DEBUG_MODULE, "[%s] Located start of SNMP response (%d bytes).", MODULE_NAME, nLength);
@@ -497,7 +499,7 @@ int processResponse(int nReceiveBufferSize, unsigned char* bufReceive, int *nSNM
   writeError(ERR_DEBUG_MODULE, "[%s] Parsing SNMP response data.", MODULE_NAME);
  
   for (i = 0; i < nReceiveBufferSize; i++) {
-    *nSNMPLength = parseLength(nReceiveBufferSize - i, &bufReceive[i]);
+    *nSNMPLength = parseLength(&bufReceive[i]);
     if (*nSNMPLength > 0)
     {
       writeError(ERR_DEBUG_MODULE, "[%s] Located start of SNMP response (%d bytes).", MODULE_NAME, *nSNMPLength);
@@ -607,7 +609,7 @@ int sendRead(int hSocket, _SNMP_DATA* _psSessionData, char* szPassword)
   {
     writeError(ERR_ERROR, "%s failed: medusaSend was not successful", MODULE_NAME);
     free(bufSend);
-    return(FAILURE);
+    return(MSTATE_FAILURE);
   }
   free(bufSend);
 
@@ -672,7 +674,7 @@ int sendWrite(int hSocket, _SNMP_DATA* _psSessionData, char* szPassword, char* s
   memset(bufSend + sizeof(snmpv1_a) + strlen(szPassword) + 15, 14 + strlen(szLocation), 1); /* set length remaining */
   memset(bufSend + sizeof(snmpv1_a) + strlen(szPassword) + 17, 12 + strlen(szLocation), 1); /* set length remaining */
   memset(bufSend + sizeof(snmpv1_a) + strlen(szPassword) + sizeof(snmpv1_w) - 1, strlen(szLocation), 1);
-  strncpy(bufSend + sizeof(snmpv1_a) + strlen(szPassword) + sizeof(snmpv1_w), szLocation, strlen(szLocation));
+  strncpy((char *)bufSend + sizeof(snmpv1_a) + strlen(szPassword) + sizeof(snmpv1_w), szLocation, strlen(szLocation));
 
   writeError(ERR_DEBUG_MODULE, "[%s] Sending SET request for system.sysLocation.", MODULE_NAME);
   if (medusaSend(hSocket, bufSend, nSendBufferSize - 1, 0) < 0)
@@ -688,11 +690,11 @@ int sendWrite(int hSocket, _SNMP_DATA* _psSessionData, char* szPassword, char* s
 
 int receiveRequest(int hSocket, _SNMP_DATA* _psSessionData, int* nPassCount, char*** arrszPassList, char** szLocation)
 {
-  unsigned char *bufReceive, *bufReceiveTmp;
+  unsigned char *bufReceive = NULL;
+  unsigned char *bufReceiveTmp = NULL;
   int i, nReceiveBufferSize, nReceiveBufferSizeTmp;
-  int nResponse, nSNMPLength;
-  char *szPasswordTmp1 = NULL;
-  char *szPasswordTmp2 = NULL;
+  int nResponse = FAILURE;
+  int nSNMPLength;
 
   nReceiveBufferSize = 0;
   bufReceive = medusaReceiveRawDelay(hSocket, &nReceiveBufferSize, _psSessionData->nReadTimeout, _psSessionData->nReadTimeout);
