@@ -41,7 +41,7 @@
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for VNC sessions"
 #define MODULE_VERSION    "2.1"
-#define MODULE_VERSION_SVN "$Id: vnc.c 1725 2012-05-23 17:00:33Z jmk $"
+#define MODULE_VERSION_SVN "$Id: vnc.c 9217 2015-05-07 18:07:03Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -65,7 +65,7 @@
 typedef struct __VNC_DATA {
   int nMaxAuthSleep;
   int nAuthType;
-  char* szChallenge;
+  unsigned char* szChallenge;
   char* szDomain;
 } _VNC_DATA;
 
@@ -145,10 +145,10 @@ int go(sLogin* logins, int argc, char *argv[])
   memset(psSessionData, 0, sizeof(_VNC_DATA));
   psSessionData->nMaxAuthSleep = 60;
 
-  if ( !(0 <= argc <= 3) )
+  if ((argc < 0) || (argc > 2))
   {
-    // Show usage information
-    writeError(ERR_ERROR, "%s is expecting 0 parameters, but it was passed %d", MODULE_NAME, argc);
+    writeError(ERR_ERROR, "%s: Incorrect number of parameters passed to module (%d). Use \"-q\" option to display module usage.", MODULE_NAME, argc);
+    return FAILURE;
   } 
   else 
   {
@@ -204,7 +204,6 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
   int iRet;
-  unsigned char zChallenge[16];
   sConnectParams params;
   int nAngrySleep = 10;
   int bAuthAllowed = FALSE;
@@ -368,7 +367,7 @@ int initModule(sLogin* psLogin, _VNC_DATA *_psSessionData)
 void vncEncryptBytes(unsigned char *bytes, char *passwd)
 {
   unsigned char key[8];
-  int i;
+  unsigned int i;
 
   /* key is simply password padded with nulls */
   for (i = 0; i < 8; i++) {
@@ -405,13 +404,13 @@ void vncEncryptPasswdMs( unsigned char *encryptedPasswd, char *passwd )
 }
 
 /* [UltraVNC/rfb/dh.cpp] */
-uint64_t bytesToInt64(const char* const bytes) {
+uint64_t bytesToInt64(const unsigned char* const bytes) {
   uint64_t result = 0;
   int i;
 
   for (i = 0; i < 8; i++) {
     result <<= 8;
-    result += (unsigned char) bytes[i];
+    result += bytes[i];
   }
   return result;
 }
@@ -441,9 +440,9 @@ void vncEncryptBytes2(unsigned char *where, const int length, unsigned char *key
 
 int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
 {
-  char ProtocolVersion[13];
+  unsigned char ProtocolVersion[13];
   int iServerProtocolVersion;
-  char* bufReceive;
+  unsigned char* bufReceive;
   int nReceiveBufferSize = 0;
   int i = 0; 
   int nSecurityTypes = 0;
@@ -453,7 +452,7 @@ int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
  
   /* --- VNC Protocol Handshake --- */
  
-  /* Retreive server VNC protocol version */
+  /* Retrieve server VNC protocol version */
   nReceiveBufferSize = 0;
   bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
   if (bufReceive == NULL)
@@ -468,29 +467,29 @@ int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
   **   RealVNC: Too many security failures
   **   WinVNC (<=3.3.3r2): Too many authentication failures
   */
-  if ((strncmp(bufReceive + 20, "Too many security failures", 26) == 0) || (strncmp(bufReceive + 20, "Too many authentication failures", 32) == 0))
+  if ((strncmp((char *)bufReceive + 20, "Too many security failures", 26) == 0) || (strncmp((char *)bufReceive + 20, "Too many authentication failures", 32) == 0))
   {
     writeError(ERR_DEBUG_MODULE, "[%s] Host reported too many security failures.", MODULE_NAME);
     return SESSION_MAX_AUTH_REALVNC;
   }
   /* 3.3, 3.7 and 3.8 are the only published protocol versions (RFB Protocol v3.8 11/26/2010) */
-  else if (strncmp(bufReceive, "RFB 003.003", 11) == 0)
+  else if (strncmp((char *)bufReceive, "RFB 003.003", 11) == 0)
   {
     memcpy(ProtocolVersion, "RFB 003.003\n", 12);
     iServerProtocolVersion = 3;
   }
-  else if (strncmp(bufReceive, "RFB 003.007", 11) == 0)
+  else if (strncmp((char *)bufReceive, "RFB 003.007", 11) == 0)
   {
     memcpy(ProtocolVersion, "RFB 003.007\n", 12);
     iServerProtocolVersion = 7;
   }
-  else if (strncmp(bufReceive, "RFB 003.008", 11) == 0)
+  else if (strncmp((char *)bufReceive, "RFB 003.008", 11) == 0)
   {
     memcpy(ProtocolVersion, "RFB 003.008\n", 12);
     iServerProtocolVersion = 8;
   }
   /* RealVNC - VNC Server Enterprise Edition E4.6.3 (r66752) */
-  else if (strncmp(bufReceive, "RFB 004.001", 11) == 0)
+  else if (strncmp((char *)bufReceive, "RFB 004.001", 11) == 0)
   {
     memcpy(ProtocolVersion, "RFB 004.001\n", 12);
     iServerProtocolVersion = 8;
@@ -514,7 +513,7 @@ int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
   
   /* --- VNC Security Type Handshake --- */
   
-  /* Retreive VNC protocol authentication scheme response */
+  /* Retrieve VNC protocol authentication scheme response */
   nReceiveBufferSize = 0;
   bufReceive = medusaReceiveRaw(hSocket, &nReceiveBufferSize);
 
@@ -543,7 +542,7 @@ int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
           writeError(ERR_DEBUG_MODULE, "VNC Session Setup Failure Message: %s", bufReceive + 8);
 
         /* Server is probably in anti-brute force mode (UltraVNC) */
-        if ((nReceiveBufferSize == 42) && (strncmp(bufReceive + 8, "Your connection has been rejected.", 34) == 0)) 
+        if ((nReceiveBufferSize == 42) && (strncmp((char *)bufReceive + 8, "Your connection has been rejected.", 34) == 0)) 
           return SESSION_MAX_AUTH_ULTRAVNC;
         else
           return SESSION_FAILURE;
@@ -572,7 +571,7 @@ int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
         }
         break;
       
-      case 0xfffffffa: /* UltaVNC MS-Logon */
+      case 0xFA: /* UltaVNC MS-Logon */
         writeError(ERR_DEBUG_MODULE, "VNC Session Setup - UltraVNC MS-Logon.");
 
         if (nReceiveBufferSize == 28)
@@ -622,7 +621,7 @@ int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
         writeError(ERR_DEBUG_MODULE, "VNC Session Setup Failure Message: %s", bufReceive + 5);
 
       /* Server is probably in anti-brute force mode (UltraVNC) */
-      if (strncmp(bufReceive + 5, "Your connection has been rejected.", 34) == 0)
+      if (strncmp((char *)bufReceive + 5, "Your connection has been rejected.", 34) == 0)
         return SESSION_MAX_AUTH_ULTRAVNC;
       else
         return SESSION_FAILURE;
@@ -691,7 +690,7 @@ int vncSessionSetup(int hSocket, _VNC_DATA* _psSessionData)
           case 0x06:  /* 6: RealVNC RA2ne */
           case 0x81:  /* 129: UNIX Logon Authentication */
           case 0x82:  /* 130: External Authentication */
-            writeError(ERR_ERROR, "VNC Session Setup - RealVNC (Type %d). RealVNC native authentication mode is NOT currently supported.", szSecurityTypes[i]);
+            writeError(ERR_ERROR, "[%s] VNC Session Setup - RealVNC (Type %d). RealVNC native authentication mode is NOT currently supported.", MODULE_NAME, szSecurityTypes[i]);
             break;
 
           case 0x11:  /* 17: UltraVNC */
@@ -791,7 +790,7 @@ int sendAuthMSLogin(int hSocket, _VNC_DATA* _psSessionData, char* szLogin, char*
   int i = 0;
   
   int client_priv = 31337; /* arbitrary value -- client would typically randomly generate */ 
-  uint64_t g, p, resp, pub;
+  uint64_t g, p, resp;
   char client_pub[8];
   BIGNUM* server_pub;
 
@@ -799,7 +798,7 @@ int sendAuthMSLogin(int hSocket, _VNC_DATA* _psSessionData, char* szLogin, char*
   int dh_error;
   unsigned char *dh_secret;
 
-  char *bufSend = NULL;
+  unsigned char *bufSend = NULL;
 
   writeError(ERR_DEBUG_MODULE, "[%s] VNC Authentication - UltraVNC Microsoft Logon", MODULE_NAME);
   
@@ -862,14 +861,14 @@ int sendAuthMSLogin(int hSocket, _VNC_DATA* _psSessionData, char* szLogin, char*
 
   if ((_psSessionData->szDomain) && (strlen(_psSessionData->szDomain) + 1 + strlen(szLogin) < 256))
   { 
-    strncpy(ms_user, _psSessionData->szDomain, strlen(_psSessionData->szDomain));
-    strncat(ms_user, "\\", 1);
-    strncat(ms_user, szLogin, strlen(szLogin));
+    strncpy((char *)ms_user, _psSessionData->szDomain, strlen(_psSessionData->szDomain));
+    strncat((char *)ms_user, "\\", 1);
+    strncat((char *)ms_user, szLogin, strlen(szLogin));
   }
   else
-    strncpy(ms_user, szLogin, 256);
+    strncpy((char *)ms_user, szLogin, 256);
 
-  strncpy(ms_passwd, szPassword, 64);
+  strncpy((char *)ms_passwd, szPassword, 64);
 
   writeError(ERR_DEBUG_MODULE, "Username: %s Password: %s", ms_user, ms_passwd);
   writeErrorBin(ERR_DEBUG_MODULE, "Username: ", ms_user, 256);
@@ -904,7 +903,7 @@ int sendAuthMSLogin(int hSocket, _VNC_DATA* _psSessionData, char* szLogin, char*
 
 int sendExit(int hSocket)
 {
-  char szExit[] = { 0x00, 0x00, 0x00, 0x00, 0x05, 0x1D, 0x03, 0x20 }; 
+  unsigned char szExit[] = { 0x00, 0x00, 0x00, 0x00, 0x05, 0x1D, 0x03, 0x20 }; 
   
   writeError(ERR_DEBUG_MODULE, "[%s] Send VNC connection termination command.", MODULE_NAME);
 
@@ -919,7 +918,7 @@ int sendExit(int hSocket)
 
 int tryLogin(int hSocket, sLogin** psLogin, _VNC_DATA* _psSessionData, char* szLogin, char* szPassword)
 {
-  char* bufReceive;
+  unsigned char* bufReceive;
   int nReceiveBufferSize = 0;
   int iRet;
 
@@ -962,7 +961,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _VNC_DATA* _psSessionData, char* szL
         sendExit(hSocket);
         break;
       case 0x01:
-        if ((nReceiveBufferSize > 8) && (strstr(bufReceive + 8, "Connection rejected by user") != NULL))
+        if ((nReceiveBufferSize > 8) && (strstr((char *)bufReceive + 8, "Connection rejected by user") != NULL))
         {
           writeError(ERR_DEBUG_MODULE, "[%s] VNC Authentication - Success (User rejected connection)", MODULE_NAME);
           (*psLogin)->pErrorMsg = malloc( 40 + 1 );
